"""
Low level *Skype for Mac OS X* interface implemented using *Carbon
distributed notifications*. Uses direct *Carbon*/*CoreFoundation*
calls through the *ctypes* module.

This module handles the options that you can pass to
`Skype.__init__` for *Mac OS X* machines.

- ``RunMainLoop`` (bool) - If set to False, Skype4Py won't start the Carbon event
  loop. Otherwise it is started in a separate thread. The loop must be running for
  Skype4Py events to work properly. Set this option to False if you plan to run the
  loop yourself or if, for example, your GUI framework does it for you.

Thanks to **Eion Robb** for reversing *Skype for Mac* API protocol.
"""
__docformat__ = 'restructuredtext en'


import sys
from ctypes import *
from ctypes.util import find_library
import threading
import time
import logging

from Skype4Py.api import Command, SkypeAPIBase, \
                         timeout2float, finalize_opts
from Skype4Py.errors import SkypeAPIError
from Skype4Py.enums import *


__all__ = ['SkypeAPI']


class CFType(object):
    """Fundamental type for all CoreFoundation types.

    :see: http://developer.apple.com/documentation/CoreFoundation/Reference/CFTypeRef/
    """

    def __init__(self, init):
        self.owner = True
        if isinstance(init, CFType):
            # copy the handle and increase the use count
            self.handle = init.get_handle()
            coref.CFRetain(self)
        elif isinstance(init, c_void_p):
            self.handle = init
        else:
            raise TypeError('illegal init type: %s' % type(init))

    @classmethod
    def from_handle(cls, handle):
        if isinstance(handle, (int, long)):
            handle = c_void_p(handle)
        elif not isinstance(handle, c_void_p):
            raise TypeError('illegal handle type: %s' % type(handle))
        obj = cls(handle)
        obj.owner = False
        return obj

    def __del__(self):
        if not coref:
            return
        if self.owner:
            coref.CFRelease(self)

    def __repr__(self):
        return '%s(handle=%s)' % (self.__class__.__name__, repr(self.handle))

    def retain(self):
        if not self.owner:
            coref.CFRetain(self)
            self.owner = True

    def get_retain_count(self):
        return coref.CFGetRetainCount(self)

    def get_handle(self):
        return self.handle

    # allows passing CF types as ctypes function parameters
    _as_parameter_ = property(get_handle)


class CFString(CFType):
    """CoreFoundation string type.

    Supports Python unicode type only. String is immutable.

    :see: http://developer.apple.com/documentation/CoreFoundation/Reference/CFStringRef/
    """

    def __init__(self, init=u''):
        if isinstance(init, (str, unicode)):
            s = unicode(init).encode('utf-8')
            init = c_void_p(coref.CFStringCreateWithBytes(None,
                                    s, len(s), 0x08000100, False))
        CFType.__init__(self, init)

    def __str__(self):
        i = coref.CFStringGetLength(self)
        size = c_long()
        if coref.CFStringGetBytes(self, 0, i, 0x08000100, 0, False, None, 0, byref(size)) > 0:
            buf = create_string_buffer(size.value)
            coref.CFStringGetBytes(self, 0, i, 0x08000100, 0, False, buf, size, None)
            return buf.value
        else:
            raise UnicodeError('CFStringGetBytes() failed')

    def __unicode__(self):
        return self.__str__().decode('utf-8')

    def __len__(self):
        return coref.CFStringGetLength(self)

    def __repr__(self):
        return 'CFString(%s)' % repr(unicode(self))


class CFNumber(CFType):
    """CoreFoundation number type.

    Supports Python int type only. Number is immutable.

    :see: http://developer.apple.com/documentation/CoreFoundation/Reference/CFNumberRef/
    """

    def __init__(self, init=0):
        if isinstance(init, (int, long)):
            init = c_void_p(coref.CFNumberCreate(None, 3, byref(c_int(int(init)))))
        CFType.__init__(self, init)

    def __int__(self):
        n = c_int()
        if coref.CFNumberGetValue(self, 3, byref(n)):
            return n.value
        return 0

    def __repr__(self):
        return 'CFNumber(%s)' % repr(int(self))


class CFDictionary(CFType):
    """CoreFoundation immutable dictionary type.

    :see: http://developer.apple.com/documentation/CoreFoundation/Reference/CFDictionaryRef/
    """

    def __init__(self, init={}):
        if isinstance(init, dict):
            d = dict(init)
            keys = (c_void_p * len(d))()
            values = (c_void_p * len(d))()
            for i, (k, v) in enumerate(d.items()):
                keys[i] = k.get_handle()
                values[i] = v.get_handle()
            init = c_void_p(coref.CFDictionaryCreate(None, keys, values, len(d),
                coref.kCFTypeDictionaryKeyCallBacks, coref.kCFTypeDictionaryValueCallBacks))
        CFType.__init__(self, init)

    def get_dict(self):
        n = len(self)
        keys = (c_void_p * n)()
        values = (c_void_p * n)()
        coref.CFDictionaryGetKeysAndValues(self, keys, values)
        d = dict()
        for i in xrange(n):
            d[CFType.from_handle(keys[i])] = CFType.from_handle(values[i])
        return d

    def __getitem__(self, key):
        return CFType.from_handle(coref.CFDictionaryGetValue(self, key))

    def __len__(self):
        return coref.CFDictionaryGetCount(self)


class CFDistributedNotificationCenter(CFType):
    """CoreFoundation distributed notification center type.

    :see: http://developer.apple.com/documentation/CoreFoundation/Reference/CFNotificationCenterRef/
    """

    CFNOTIFICATIONCALLBACK = CFUNCTYPE(None, c_void_p, c_void_p, c_void_p, c_void_p, c_void_p)

    def __init__(self):
        CFType.__init__(self, c_void_p(coref.CFNotificationCenterGetDistributedCenter()))
        # there is only one distributed notification center per application
        self.owner = False
        self.callbacks = {}
        self._c_callback = self.CFNOTIFICATIONCALLBACK(self._callback)

    def _callback(self, center, observer, name, obj, userInfo):
        observer = CFString.from_handle(observer)
        name = CFString.from_handle(name)
        if obj:
            obj = CFString.from_handle(obj)
        userInfo = CFDictionary.from_handle(userInfo)
        callback = self.callbacks[(unicode(observer), unicode(name))]
        callback(self, observer, name, obj, userInfo)

    def add_observer(self, observer, callback, name=None, obj=None,
            drop=False, coalesce=False, hold=False, immediate=False):
        if not callable(callback):
            raise TypeError('callback must be callable')
        observer = CFString(observer)
        self.callbacks[(unicode(observer), unicode(name))] = callback
        if name is not None:
            name = CFString(name)
        if obj is not None:
            obj = CFString(obj)
        if drop:
            behaviour = 1
        elif coalesce:
            behaviour = 2
        elif hold:
            behaviour = 3
        elif immediate:
            behaviour = 4
        else:
            behaviour = 0
        coref.CFNotificationCenterAddObserver(self, observer,
                self._c_callback, name, obj, behaviour)

    def remove_observer(self, observer, name=None, obj=None):
        observer = CFString(observer)
        if name is not None:
            name = CFString(name)
        if obj is not None:
            obj = CFString(obj)
        coref.CFNotificationCenterRemoveObserver(self, observer, name, obj)
        try:
            del self.callbacks[(unicode(observer), unicode(name))]
        except KeyError:
            pass

    def post_notification(self, name, obj=None, userInfo=None, immediate=False):
        name = CFString(name)
        if obj is not None:
            obj = CFString(obj)
        if userInfo is not None:
            userInfo = CFDictionary(userInfo)
        coref.CFNotificationCenterPostNotification(self, name, obj, userInfo, immediate)


class EventLoop(object):
    """Carbon event loop object for the current thread.
    
    The Carbon reference documentation seems to be gone from developer.apple.com, the following
    link points to a mirror I found. I don't know how long until this one is gone too.
    
    :see: http://www.monen.nl/DevDoc/documentation/Carbon/Reference/Carbon_Event_Manager_Ref/index.html
    """
    
    def __init__(self):
        self.handle = c_void_p(carbon.GetCurrentEventLoop())

    @staticmethod
    def run(timeout=-1):
        # Timeout is expressed in seconds (float), -1 means forever.
        # Returns True if aborted (eventLoopQuitErr).
        return (carbon.RunCurrentEventLoop(timeout) == -9876)

    def stop(self):
        carbon.QuitEventLoop(self.handle)


# load the Carbon and CoreFoundation frameworks
# (only if not building the docs)
if not getattr(sys, 'skype4py_setup', False):

    path = find_library('Carbon')
    if path is None:
        raise ImportError('Could not find Carbon.framework')
    carbon = cdll.LoadLibrary(path)
    carbon.RunCurrentEventLoop.argtypes = (c_double,)

    path = find_library('CoreFoundation')
    if path is None:
        raise ImportError('Could not find CoreFoundation.framework')
    coref = cdll.LoadLibrary(path)


class SkypeAPI(SkypeAPIBase):
    """
    :note: Code based on Pidgin Skype Plugin source
           (http://code.google.com/p/skype4pidgin/).
           Permission to use granted by the author.
    """

    def __init__(self, opts):
        self.logger = logging.getLogger('Skype4Py.api.darwin.SkypeAPI')
        SkypeAPIBase.__init__(self)
        self.run_main_loop = opts.pop('RunMainLoop', True)
        finalize_opts(opts)
        self.center = CFDistributedNotificationCenter()
        self.is_available = False
        self.client_id = -1
        self.thread_started = False

    def start(self):
        """
        Start the thread associated with this API object.
        Ensure that the call is made no more than once,
        to avoid raising a RuntimeError.
        """
        if not self.thread_started:
            super(SkypeAPI, self).start()
            self.thread_started = True

    def run(self):
        self.logger.info('thread started')
        if self.run_main_loop:
            self.loop = EventLoop()
            EventLoop.run()
        self.logger.info('thread finished')

    def close(self):
        if hasattr(self, 'loop'):
            self.loop.stop()
            self.client_id = -1
        SkypeAPIBase.close(self)

    def set_friendly_name(self, friendly_name):
        SkypeAPIBase.set_friendly_name(self, friendly_name)
        if self.attachment_status == apiAttachSuccess:
            # reattach with the new name
            self.set_attachment_status(apiAttachUnknown)
            self.attach()

    def attach(self, timeout, wait=True):
        if self.attachment_status in (apiAttachPendingAuthorization, apiAttachSuccess):
            return
        self.acquire()
        try:
            try:
                self.start()
            except AssertionError:
                pass
            t = threading.Timer(timeout2float(timeout), lambda: setattr(self, 'wait', False))
            try:
                self.init_observer()
                self.client_id = -1
                self.set_attachment_status(apiAttachPendingAuthorization)
                self.post('SKSkypeAPIAttachRequest')
                self.wait = True
                if wait:
                    t.start()
                while self.wait and self.attachment_status == apiAttachPendingAuthorization:
                    if self.run_main_loop:
                        time.sleep(1.0)
                    else:
                        EventLoop.run(1.0)
            finally:
                t.cancel()
            if not self.wait:
                self.set_attachment_status(apiAttachUnknown)
                raise SkypeAPIError('Skype attach timeout')
        finally:
            self.release()
        command = Command('PROTOCOL %s' % self.protocol, Blocking=True)
        self.send_command(command)
        self.protocol = int(command.Reply.rsplit(None, 1)[-1])

    def is_running(self):
        try:
            self.start()
        except AssertionError:
            pass
        self.init_observer()
        self.is_available = False
        self.post('SKSkypeAPIAvailabilityRequest')
        time.sleep(1.0)
        return self.is_available

    def startup(self, minimized, nosplash):
        if not self.is_running():
            from subprocess import Popen
            nul = file('/dev/null')
            Popen(['/Applications/Skype.app/Contents/MacOS/Skype'], stdin=nul, stdout=nul, stderr=nul)

    def send_command(self, command):
        if not self.attachment_status == apiAttachSuccess:
            self.attach(command.Timeout)
        self.push_command(command)
        self.notifier.sending_command(command)
        cmd = u'#%d %s' % (command.Id, command.Command)
        if command.Blocking:
            if self.run_main_loop:
                command._event = event = threading.Event()
            else:
                command._loop = EventLoop()
        else:
            command._timer = timer = threading.Timer(command.timeout2float(), self.pop_command, (command.Id,))

        self.logger.debug('sending %s', repr(cmd))
        userInfo = CFDictionary({CFString('SKYPE_API_COMMAND'): CFString(cmd),
                                 CFString('SKYPE_API_CLIENT_ID'): CFNumber(self.client_id)})
        self.post('SKSkypeAPICommand', userInfo)

        if command.Blocking:
            if self.run_main_loop:
                event.wait(command.timeout2float())
                if not event.isSet():
                    raise SkypeAPIError('Skype command timeout')
            else:
                if not EventLoop.run(command.timeout2float()):
                    raise SkypeAPIError('Skype command timeout')
        else:
            timer.start()

    def init_observer(self):
        if self.has_observer():
            self.delete_observer()
        self.observer = CFString(self.friendly_name)
        self.center.add_observer(self.observer, self.SKSkypeAPINotification, 'SKSkypeAPINotification', immediate=True)
        self.center.add_observer(self.observer, self.SKSkypeWillQuit, 'SKSkypeWillQuit', immediate=True)
        self.center.add_observer(self.observer, self.SKSkypeBecameAvailable, 'SKSkypeBecameAvailable', immediate=True)
        self.center.add_observer(self.observer, self.SKAvailabilityUpdate, 'SKAvailabilityUpdate', immediate=True)
        self.center.add_observer(self.observer, self.SKSkypeAttachResponse, 'SKSkypeAttachResponse', immediate=True)

    def delete_observer(self):
        if not self.has_observer():
            return
        self.center.remove_observer(self.observer, 'SKSkypeAPINotification')
        self.center.remove_observer(self.observer, 'SKSkypeWillQuit')
        self.center.remove_observer(self.observer, 'SKSkypeBecameAvailable')
        self.center.remove_observer(self.observer, 'SKAvailabilityUpdate')
        self.center.remove_observer(self.observer, 'SKSkypeAttachResponse')
        del self.observer

    def has_observer(self):
        return hasattr(self, 'observer')

    def post(self, name, userInfo=None):
        if not self.has_observer():
            self.init_observer()
        self.center.post_notification(name, self.observer, userInfo, immediate=True)

    def SKSkypeAPINotification(self, center, observer, name, obj, userInfo):
        client_id = int(CFNumber(userInfo[CFString('SKYPE_API_CLIENT_ID')]))
        if client_id != 999 and (client_id == 0 or client_id != self.client_id):
            return
        cmd = unicode(CFString(userInfo[CFString('SKYPE_API_NOTIFICATION_STRING')]))
        self.logger.debug('received %s', repr(cmd))

        if cmd.startswith(u'#'):
            p = cmd.find(u' ')
            command = self.pop_command(int(cmd[1:p]))
            if command is not None:
                command.Reply = cmd[p + 1:]
                if command.Blocking:
                    if self.run_main_loop:
                        command._event.set()
                    else:
                        command._loop.stop()
                else:
                    command._timer.cancel()
                self.notifier.reply_received(command)
            else:
                self.notifier.notification_received(cmd[p + 1:])
        else:
            self.notifier.notification_received(cmd)

    def SKSkypeWillQuit(self, center, observer, name, obj, userInfo):
        self.logger.debug('received SKSkypeWillQuit')
        self.set_attachment_status(apiAttachNotAvailable)

    def SKSkypeBecameAvailable(self, center, observer, name, obj, userInfo):
        self.logger.debug('received SKSkypeBecameAvailable')
        self.set_attachment_status(apiAttachAvailable)

    def SKAvailabilityUpdate(self, center, observer, name, obj, userInfo):
        self.logger.debug('received SKAvailabilityUpdate')
        self.is_available = not not int(CFNumber(userInfo[CFString('SKYPE_API_AVAILABILITY')]))

    def SKSkypeAttachResponse(self, center, observer, name, obj, userInfo):
        self.logger.debug('received SKSkypeAttachResponse')
        # It seems that this notification is not called if the access is refused. Therefore we can't
        # distinguish between attach timeout and access refuse.
        if unicode(CFString(userInfo[CFString('SKYPE_API_CLIENT_NAME')])) == self.friendly_name:
            response = int(CFNumber(userInfo[CFString('SKYPE_API_ATTACH_RESPONSE')]))
            if response and self.client_id == -1:
                self.client_id = response
                self.set_attachment_status(apiAttachSuccess)



apiAttachAvailable = u'API \u05d6\u05de\u05d9\u05df'
apiAttachNotAvailable = u'\u05d0\u05d9\u05e0\u05d5 \u05d6\u05de\u05d9\u05df'
apiAttachPendingAuthorization = u'\u05de\u05de\u05ea\u05d9\u05df \u05dc\u05d0\u05d9\u05e9\u05d5\u05e8'
apiAttachRefused = u'\u05e0\u05d3\u05d7\u05d4'
apiAttachSuccess = u'\u05d4\u05e6\u05dc\u05d7\u05d4'
apiAttachUnknown = u'\u05dc\u05d0 \u05d9\u05d3\u05d5\u05e2'
budDeletedFriend = u'\u05e0\u05de\u05d7\u05e7 \u05de\u05e8\u05e9\u05d9\u05de\u05ea \u05d7\u05d1\u05e8\u05d9\u05dd'
budFriend = u'\u05d7\u05d1\u05e8'
budNeverBeenFriend = u'\u05dc\u05e2\u05d5\u05dc\u05dd \u05dc\u05d0 \u05d4\u05d9\u05d4 \u05d1\u05e8\u05e9\u05d9\u05de\u05ea \u05d7\u05d1\u05e8\u05d9\u05dd'
budPendingAuthorization = u'\u05de\u05de\u05ea\u05d9\u05df \u05dc\u05d0\u05d9\u05e9\u05d5\u05e8'
budUnknown = u'\u05dc\u05d0 \u05d9\u05d3\u05d5\u05e2'
cfrBlockedByRecipient = u'\u05d4\u05e9\u05d9\u05d7\u05d4 \u05e0\u05d7\u05e1\u05de\u05d4 \u05e2\u05dc \u05d9\u05d3\u05d9 \u05d4\u05de\u05e7\u05d1\u05dc'
cfrMiscError = u'\u05e9\u05d2\u05d9\u05d0\u05d4 \u05e9\u05d5\u05e0\u05d4'
cfrNoCommonCodec = u'\u05d0\u05d9\u05df \u05e7\u05d9\u05d3\u05d5\u05d3'
cfrNoProxyFound = u'\u05dc\u05d0 \u05e0\u05de\u05e6\u05d0 \u05e4\u05e8\u05d5\u05e7\u05e1\u05d9'
cfrNotAuthorizedByRecipient = u'\u05d4\u05de\u05e9\u05ea\u05de\u05e9 \u05d4\u05e0\u05d5\u05db\u05d7\u05d9 \u05d0\u05d9\u05e0\u05d5 \u05de\u05d0\u05d5\u05e9\u05e8 \u05e2\u05dc \u05d9\u05d3\u05d9 \u05d4\u05de\u05e7\u05d1\u05dc'
cfrRecipientNotFriend = u'\u05d4\u05de\u05e7\u05d1\u05dc \u05d0\u05d9\u05e0\u05d5 \u05d7\u05d1\u05e8'
cfrRemoteDeviceError = u'\u05e9\u05d2\u05d9\u05d0\u05d4 \u05d1\u05d4\u05ea\u05e7\u05df \u05e6\u05dc\u05d9\u05dc \u05e9\u05dc\u05d0 \u05d1\u05de\u05d7\u05e9\u05d1 \u05d6\u05d4'
cfrSessionTerminated = u'\u05de\u05e4\u05d2\u05e9 \u05d4\u05e1\u05ea\u05d9\u05d9\u05dd'
cfrSoundIOError = u'\u05e9\u05d2\u05d9\u05d0\u05ea \u05e7\u05d5\u05dc I/O'
cfrSoundRecordingError = u'\u05e9\u05d2\u05d9\u05d0\u05d4 \u05d1\u05d4\u05e7\u05dc\u05d8\u05ea \u05e7\u05d5\u05dc'
cfrUnknown = u'\u05dc\u05d0 \u05d9\u05d3\u05d5\u05e2'
cfrUserDoesNotExist = u'\u05de\u05e9\u05ea\u05de\u05e9/\u05de\u05e1\u05e4\u05e8 \u05d8\u05dc\u05e4\u05d5\u05df \u05d0\u05d9\u05e0\u05d5 \u05e7\u05d9\u05d9\u05dd'
cfrUserIsOffline = u'\u05d4\u05d9\u05d0 \u05d0\u05d5 \u05d4\u05d5\u05d0 \u05dc\u05d0 \u05de\u05d7\u05d5\u05d1\u05e8\u05d9\u05dd'
chsAllCalls = u'\u05ea\u05d9\u05d1\u05ea \u05d3\u05d5-\u05e9\u05d9\u05d7 \u05e7\u05d5\u05d3\u05de\u05ea'
chsDialog = u'\u05ea\u05d9\u05d1\u05ea \u05d3\u05d5-\u05e9\u05d9\u05d7'
chsIncomingCalls = u"\u05e6\u05d5\u05e8\u05da \u05d1\u05d0\u05d9\u05e9\u05d5\u05e8 \u05e6'\u05d8 \u05de\u05e8\u05d5\u05d1\u05d4 \u05de\u05e9\u05ea\u05ea\u05e4\u05d9\u05dd"
chsLegacyDialog = u'\u05ea\u05d9\u05d1\u05ea \u05d3\u05d5-\u05e9\u05d9\u05d7 \u05e7\u05d5\u05d3\u05de\u05ea'
chsMissedCalls = u'\u05ea\u05d9\u05d1\u05ea \u05d3\u05d5-\u05e9\u05d9\u05d7'
chsMultiNeedAccept = u"\u05e6\u05d5\u05e8\u05da \u05d1\u05d0\u05d9\u05e9\u05d5\u05e8 \u05e6'\u05d8 \u05de\u05e8\u05d5\u05d1\u05d4 \u05de\u05e9\u05ea\u05ea\u05e4\u05d9\u05dd"
chsMultiSubscribed = u"\u05e6'\u05d8 \u05de\u05e8\u05d5\u05d1\u05d4 \u05de\u05e9\u05ea\u05ea\u05e4\u05d9\u05dd"
chsOutgoingCalls = u"\u05e6'\u05d8 \u05de\u05e8\u05d5\u05d1\u05d4 \u05de\u05e9\u05ea\u05ea\u05e4\u05d9\u05dd"
chsUnknown = u'\u05dc\u05d0 \u05d9\u05d3\u05d5\u05e2'
chsUnsubscribed = u'\u05d1\u05d9\u05d8\u05d5\u05dc \u05d4\u05e9\u05ea\u05ea\u05e4\u05d5\u05ea'
clsBusy = u'\u05ea\u05e4\u05d5\u05e1'
clsCancelled = u'\u05d1\u05d5\u05d8\u05dc'
clsEarlyMedia = u'\u05e0\u05d2\u05df Early Media'
clsFailed = u'\u05d4\u05e9\u05d9\u05d7\u05d4 \u05e0\u05db\u05e9\u05dc\u05d4'
clsFinished = u'\u05d4\u05e1\u05ea\u05d9\u05d9\u05de\u05d4'
clsInProgress = u'\u05e9\u05d9\u05d7\u05d4 \u05de\u05ea\u05e7\u05d9\u05d9\u05de\u05ea'
clsLocalHold = u'\u05d1\u05d4\u05de\u05ea\u05e0\u05d4'
clsMissed = u'\u05e9\u05d9\u05d7\u05d4 \u05e9\u05dc\u05d0 \u05e0\u05e2\u05e0\u05ea\u05d4'
clsOnHold = u'\u05de\u05d7\u05d6\u05d9\u05e7'
clsRefused = u'\u05e0\u05d3\u05d7\u05d4'
clsRemoteHold = u'\u05d1\u05d4\u05de\u05ea\u05e0\u05d4'
clsRinging = u'\u05de\u05d7\u05d9\u05d9\u05d2'
clsRouting = u'\u05de\u05e0\u05ea\u05d1'
clsTransferred = u'\u05dc\u05d0 \u05d9\u05d3\u05d5\u05e2'
clsTransferring = u'\u05dc\u05d0 \u05d9\u05d3\u05d5\u05e2'
clsUnknown = u'\u05dc\u05d0 \u05d9\u05d3\u05d5\u05e2'
clsUnplaced = u'\u05dc\u05d0 \u05d1\u05d5\u05e6\u05e2'
clsVoicemailBufferingGreeting = u'\u05d7\u05d5\u05e6\u05e5 \u05d4\u05d5\u05d3\u05e2\u05ea \u05e4\u05ea\u05d9\u05d7\u05d4'
clsVoicemailCancelled = u'\u05d4\u05d5\u05d3\u05e2\u05d4 \u05e7\u05d5\u05dc\u05d9\u05ea \u05d1\u05d5\u05d8\u05dc\u05d4'
clsVoicemailFailed = u'\u05d4\u05d4\u05d5\u05d3\u05e2\u05d4 \u05e0\u05db\u05e9\u05dc\u05d4'
clsVoicemailPlayingGreeting = u'\u05de\u05e9\u05de\u05d9\u05e2 \u05d4\u05d5\u05d3\u05e2\u05ea \u05e4\u05ea\u05d9\u05d7\u05d4'
clsVoicemailRecording = u'\u05d4\u05e7\u05dc\u05d8\u05ea \u05d4\u05d5\u05d3\u05e2\u05d4 \u05e7\u05d5\u05dc\u05d9\u05ea'
clsVoicemailSent = u'\u05d4\u05d5\u05d3\u05e2\u05d4 \u05e7\u05d5\u05dc\u05d9\u05ea \u05e0\u05e9\u05dc\u05d7\u05d4'
clsVoicemailUploading = u'\u05e9\u05d5\u05dc\u05d7 \u05d4\u05d5\u05d3\u05e2\u05d4 \u05e7\u05d5\u05dc\u05d9\u05ea'
cltIncomingP2P = u'\u05e9\u05d9\u05d7\u05ea \u05e2\u05de\u05d9\u05ea-\u05dc\u05e2\u05de\u05d9\u05ea \u05e0\u05db\u05e0\u05e1\u05ea'
cltIncomingPSTN = u'\u05e9\u05d9\u05d7\u05ea \u05d8\u05dc\u05e4\u05d5\u05df \u05e0\u05db\u05e0\u05e1\u05ea'
cltOutgoingP2P = u'\u05e9\u05d9\u05d7\u05ea \u05e2\u05de\u05d9\u05ea-\u05dc\u05e2\u05de\u05d9\u05ea \u05d9\u05d5\u05e6\u05d0\u05ea'
cltOutgoingPSTN = u'\u05e9\u05d9\u05d7\u05ea \u05d8\u05dc\u05e4\u05d5\u05df \u05d9\u05d5\u05e6\u05d0\u05ea'
cltUnknown = u'\u05dc\u05d0 \u05d9\u05d3\u05d5\u05e2'
cmeAddedMembers = u'\u05d7\u05d1\u05e8\u05d9\u05dd \u05e9\u05d4\u05ea\u05d5\u05d5\u05e1\u05e4\u05d5'
cmeCreatedChatWith = u"\u05e6\u05d5\u05e8 \u05e6'\u05d8 \u05e2\u05dd"
cmeEmoted = u'\u05dc\u05d0 \u05d9\u05d3\u05d5\u05e2'
cmeLeft = u'\u05e2\u05d6\u05d1'
cmeSaid = u'Said'
cmeSawMembers = u'\u05d7\u05d1\u05e8\u05d9\u05dd \u05e9\u05e0\u05e8\u05d0\u05d5'
cmeSetTopic = u'\u05e7\u05d1\u05e2 \u05e0\u05d5\u05e9\u05d0'
cmeUnknown = u'\u05dc\u05d0 \u05d9\u05d3\u05d5\u05e2'
cmsRead = u'\u05e0\u05e7\u05e8\u05d0\u05d4'
cmsReceived = u'\u05e0\u05ea\u05e7\u05d1\u05dc\u05d4'
cmsSending = u'....\u05e9\u05d5\u05dc\u05d7'
cmsSent = u'\u05e0\u05e9\u05dc\u05d7\u05d4'
cmsUnknown = u'\u05dc\u05d0 \u05d9\u05d3\u05d5\u05e2'
conConnecting = u'\u05de\u05ea\u05d7\u05d1\u05e8'
conOffline = u'\u05de\u05e0\u05d5\u05ea\u05e7'
conOnline = u'\u05de\u05d7\u05d5\u05d1\u05e8'
conPausing = u'\u05d4\u05e4\u05e1\u05e7\u05d4'
conUnknown = u'\u05dc\u05d0 \u05d9\u05d3\u05d5\u05e2'
cusAway = u'\u05dc\u05d0 \u05e0\u05de\u05e6\u05d0'
cusDoNotDisturb = u'\u05e0\u05d0 \u05dc\u05d0 \u05dc\u05d4\u05e4\u05e8\u05d9\u05e2'
cusInvisible = u'\u05d1\u05dc\u05ea\u05d9 \u05e0\u05e8\u05d0\u05d4'
cusLoggedOut = u'\u05de\u05e0\u05d5\u05ea\u05e7'
cusNotAvailable = u'\u05d0\u05d9\u05e0\u05d5 \u05d6\u05de\u05d9\u05df'
cusOffline = u'\u05de\u05e0\u05d5\u05ea\u05e7'
cusOnline = u'\u05de\u05d7\u05d5\u05d1\u05e8'
cusSkypeMe = u'\u05d3\u05d1\u05e8 \u05d0\u05ea\u05d9'
cusUnknown = u'\u05dc\u05d0 \u05d9\u05d3\u05d5\u05e2'
cvsBothEnabled = u'\u05e9\u05dc\u05d7 \u05d5\u05e7\u05d1\u05dc \u05d5\u05d9\u05d3\u05d0\u05d5'
cvsNone = u'\u05d0\u05d9\u05df \u05d5\u05d9\u05d3\u05d0\u05d5'
cvsReceiveEnabled = u'\u05d5\u05d9\u05d3\u05d0\u05d5 \u05d4\u05ea\u05e7\u05d1\u05dc'
cvsSendEnabled = u'\u05d5\u05d9\u05d3\u05d0\u05d5 \u05e0\u05e9\u05dc\u05d7'
cvsUnknown = u''
grpAllFriends = u'\u05db\u05dc \u05d4\u05d7\u05d1\u05e8\u05d9\u05dd'
grpAllUsers = u'\u05db\u05dc \u05d4\u05de\u05e9\u05ea\u05de\u05e9\u05d9\u05dd'
grpCustomGroup = u'\u05de\u05d5\u05ea\u05d0\u05dd \u05d0\u05d9\u05e9\u05d9\u05ea'
grpOnlineFriends = u'\u05d7\u05d1\u05e8\u05d9\u05dd \u05de\u05d7\u05d5\u05d1\u05e8\u05d9\u05dd'
grpPendingAuthorizationFriends = u'\u05de\u05de\u05ea\u05d9\u05df \u05dc\u05d0\u05d9\u05e9\u05d5\u05e8'
grpProposedSharedGroup = u'Proposed Shared Group'
grpRecentlyContactedUsers = u'\u05de\u05e9\u05ea\u05de\u05e9\u05d9\u05dd \u05d0\u05d9\u05ea\u05dd \u05e0\u05d5\u05e6\u05e8 \u05e7\u05e9\u05e8 \u05dc\u05d0\u05d7\u05e8\u05d5\u05e0\u05d4'
grpSharedGroup = u'Shared Group'
grpSkypeFriends = u'\u05d7\u05d1\u05e8\u05d9 \u05e1\u05e7\u05d9\u05d9\u05e4'
grpSkypeOutFriends = u'\u05d7\u05d1\u05e8\u05d9 SkypeOut'
grpUngroupedFriends = u'\u05d7\u05d1\u05e8\u05d9\u05dd \u05dc\u05dc\u05d0 \u05e7\u05d1\u05d5\u05e6\u05d4'
grpUnknown = u'\u05dc\u05d0 \u05d9\u05d3\u05d5\u05e2'
grpUsersAuthorizedByMe = u'\u05d0\u05d5\u05e9\u05e8 \u05e2\u05dc \u05d9\u05d3\u05d9'
grpUsersBlockedByMe = u'\u05e0\u05d7\u05e1\u05dd \u05e2\u05dc \u05d9\u05d3\u05d9'
grpUsersWaitingMyAuthorization = u'\u05de\u05de\u05ea\u05d9\u05df \u05dc\u05d0\u05d9\u05e9\u05d5\u05e8\u05d9'
leaAddDeclined = u'\u05d4\u05d1\u05e7\u05e9\u05d4 \u05e0\u05d3\u05d7\u05ea\u05d4'
leaAddedNotAuthorized = u'\u05d4\u05de\u05d5\u05e1\u05d9\u05e3 \u05d7\u05d9\u05d9\u05d1 \u05dc\u05d4\u05d9\u05d5\u05ea \u05de\u05d0\u05d5\u05e9\u05e8'
leaAdderNotFriend = u'\u05d4\u05de\u05d5\u05e1\u05d9\u05e3 \u05d7\u05d9\u05d9\u05d1 \u05dc\u05d4\u05d9\u05d5\u05ea \u05d7\u05d1\u05e8'
leaUnknown = u'\u05dc\u05d0 \u05d9\u05d3\u05d5\u05e2'
leaUnsubscribe = u'\u05d1\u05d9\u05d8\u05d5\u05dc \u05d4\u05e9\u05ea\u05ea\u05e4\u05d5\u05ea'
leaUserIncapable = u'\u05de\u05e9\u05ea\u05de\u05e9 \u05dc\u05d0 \u05de\u05ea\u05d0\u05d9\u05dd'
leaUserNotFound = u'\u05de\u05e9\u05ea\u05de\u05e9 \u05dc\u05d0 \u05e0\u05de\u05e6\u05d0'
olsAway = u'\u05dc\u05d0 \u05e0\u05de\u05e6\u05d0'
olsDoNotDisturb = u'\u05e0\u05d0 \u05dc\u05d0 \u05dc\u05d4\u05e4\u05e8\u05d9\u05e2'
olsNotAvailable = u'\u05d0\u05d9\u05e0\u05d5 \u05d6\u05de\u05d9\u05df'
olsOffline = u'\u05de\u05e0\u05d5\u05ea\u05e7'
olsOnline = u'\u05de\u05d7\u05d5\u05d1\u05e8'
olsSkypeMe = u'\u05d3\u05d1\u05e8 \u05d0\u05ea\u05d9'
olsSkypeOut = u'SkypeOut'
olsUnknown = u'\u05dc\u05d0 \u05d9\u05d3\u05d5\u05e2'
smsMessageStatusComposing = u'Composing'
smsMessageStatusDelivered = u'Delivered'
smsMessageStatusFailed = u'Failed'
smsMessageStatusRead = u'Read'
smsMessageStatusReceived = u'Received'
smsMessageStatusSendingToServer = u'Sending to Server'
smsMessageStatusSentToServer = u'Sent to Server'
smsMessageStatusSomeTargetsFailed = u'Some Targets Failed'
smsMessageStatusUnknown = u'Unknown'
smsMessageTypeCCRequest = u'Confirmation Code Request'
smsMessageTypeCCSubmit = u'Confirmation Code Submit'
smsMessageTypeIncoming = u'Incoming'
smsMessageTypeOutgoing = u'Outgoing'
smsMessageTypeUnknown = u'Unknown'
smsTargetStatusAcceptable = u'Acceptable'
smsTargetStatusAnalyzing = u'Analyzing'
smsTargetStatusDeliveryFailed = u'Delivery Failed'
smsTargetStatusDeliveryPending = u'Delivery Pending'
smsTargetStatusDeliverySuccessful = u'Delivery Successful'
smsTargetStatusNotRoutable = u'Not Routable'
smsTargetStatusUndefined = u'Undefined'
smsTargetStatusUnknown = u'Unknown'
usexFemale = u'\u05e0\u05e7\u05d1\u05d4'
usexMale = u'\u05d6\u05db\u05e8'
usexUnknown = u'\u05dc\u05d0 \u05d9\u05d3\u05d5\u05e2'
vmrConnectError = u'\u05e9\u05d2\u05d9\u05d0\u05ea \u05d7\u05d9\u05d1\u05d5\u05e8'
vmrFileReadError = u'\u05e9\u05d2\u05d9\u05d0\u05d4 \u05d1\u05e7\u05e8\u05d9\u05d0\u05ea \u05d4\u05e7\u05d5\u05d1\u05e5'
vmrFileWriteError = u'\u05e9\u05d2\u05d9\u05d0\u05d4 \u05d1\u05db\u05ea\u05d9\u05d1\u05d4 \u05dc\u05e7\u05d5\u05d1\u05e5'
vmrMiscError = u'\u05e9\u05d2\u05d9\u05d0\u05d4 \u05e9\u05d5\u05e0\u05d4'
vmrNoError = u'\u05d0\u05d9\u05df \u05e9\u05d2\u05d9\u05d0\u05d4'
vmrNoPrivilege = u'\u05dc\u05d0 \u05d6\u05db\u05d0\u05d9 \u05dc\u05ea\u05d0 \u05e7\u05d5\u05dc\u05d9'
vmrNoVoicemail = u'\u05ea\u05d0 \u05e7\u05d5\u05dc\u05d9 \u05dc\u05d0 \u05e7\u05d9\u05d9\u05dd'
vmrPlaybackError = u'\u05e9\u05d2\u05d9\u05d0\u05ea \u05d4\u05e9\u05de\u05e2\u05d4'
vmrRecordingError = u'\u05e9\u05d2\u05d9\u05d0\u05ea \u05d4\u05e7\u05dc\u05d8\u05d4'
vmrUnknown = u'\u05dc\u05d0 \u05d9\u05d3\u05d5\u05e2'
vmsBlank = u'\u05e8\u05d9\u05e7\u05d4'
vmsBuffering = u'\u05d7\u05e6\u05d9\u05e6\u05d4'
vmsDeleting = u'\u05e0\u05de\u05d7\u05e7\u05ea'
vmsDownloading = u'\u05d4\u05d5\u05e8\u05d3\u05d4'
vmsFailed = u'\u05e0\u05db\u05e9\u05dc\u05d4'
vmsNotDownloaded = u'\u05dc\u05d0 \u05d4\u05d5\u05e8\u05d3\u05d4'
vmsPlayed = u'\u05d4\u05d5\u05e9\u05de\u05e2\u05d4'
vmsPlaying = u'\u05de\u05d5\u05e9\u05de\u05e2\u05ea'
vmsRecorded = u'\u05d4\u05d5\u05e7\u05dc\u05d8\u05d4'
vmsRecording = u'\u05d4\u05e7\u05dc\u05d8\u05ea \u05d4\u05d5\u05d3\u05e2\u05d4 \u05e7\u05d5\u05dc\u05d9\u05ea'
vmsUnknown = u'\u05dc\u05d0 \u05d9\u05d3\u05d5\u05e2'
vmsUnplayed = u'\u05dc\u05d0 \u05d4\u05d5\u05e9\u05de\u05e2\u05d4'
vmsUploaded = u'\u05e0\u05e9\u05dc\u05d7\u05d4'
vmsUploading = u'\u05e0\u05e9\u05dc\u05d7\u05ea'
vmtCustomGreeting = u'\u05d4\u05d5\u05d3\u05e2\u05ea \u05e4\u05ea\u05d9\u05d7\u05d4 \u05de\u05d5\u05ea\u05d0\u05de\u05ea \u05d0\u05d9\u05e9\u05d9\u05ea'
vmtDefaultGreeting = u'\u05d4\u05d5\u05d3\u05e2\u05ea \u05e4\u05ea\u05d9\u05d7\u05d4 - \u05d1\u05e8\u05d9\u05e8\u05ea \u05de\u05d7\u05d3\u05dc'
vmtIncoming = u'\u05e0\u05db\u05e0\u05e1\u05ea \u05d4\u05d5\u05d3\u05e2\u05d4 \u05e7\u05d5\u05dc\u05d9\u05ea'
vmtOutgoing = u'\u05d9\u05d5\u05e6\u05d0\u05ea'
vmtUnknown = u'\u05dc\u05d0 \u05d9\u05d3\u05d5\u05e2'
vssAvailable = u'\u05d6\u05de\u05d9\u05e0\u05d4'
vssNotAvailable = u'\u05d0\u05d9\u05e0\u05d5 \u05d6\u05de\u05d9\u05df'
vssPaused = u'\u05d4\u05e4\u05e1\u05e7\u05d4'
vssRejected = u'\u05e0\u05d3\u05d7\u05d4'
vssRunning = u'\u05e4\u05d5\u05e2\u05dc\u05ea'
vssStarting = u'\u05d4\u05ea\u05d7\u05dc'
vssStopping = u'\u05e2\u05e6\u05d9\u05e8\u05d4'
vssUnknown = u'\u05dc\u05d0 \u05d9\u05d3\u05d5\u05e2'



"""
Skype4Py is a multiplatform Skype API wrapper for Python.
"""
__docformat__ = 'restructuredtext en'


from skype import Skype
from callchannel import CallChannelManager
from errors import SkypeError, SkypeAPIError
from enums import *
from api import platform
import logging


"""The version of Skype4Py."""


class NullHandler(logging.Handler):
    def emit(self, record):
        pass


# Suppress the "No handlers could be found for logger (...)" message.
logging.getLogger('Skype4Py').addHandler(NullHandler())



"""Error classes.
"""
__docformat__ = 'restructuredtext en'


class SkypeAPIError(Exception):
    """Exception raised whenever there is a problem with connection between
    Skype4Py and Skype client. It can be subscripted in which case following
    information can be obtained:

    +-------+------------------------------+
    | Index | Meaning                      |
    +=======+==============================+
    |     0 | (unicode) Error description. |
    +-------+------------------------------+
    """

    def __init__(self, errstr):
        """__init__.

        :Parameters:
          errstr : unicode
            Error description.
        """
        Exception.__init__(self, str(errstr))


class SkypeError(Exception):
    """Raised whenever Skype client reports an error back to Skype4Py. It can be
    subscripted in which case following information can be obtained:

    +-------+------------------------------+
    | Index | Meaning                      |
    +=======+==============================+
    |     0 | (int) Error code. See below. |
    +-------+------------------------------+
    |     1 | (unicode) Error description. |
    +-------+------------------------------+

    :see: https://developer.skype.com/Docs/ApiDoc/Error_codes for more information about
          Skype error codes. Additionally an **error code 0** can be raised by Skype4Py
          itself.
    """

    def __init__(self, errno, errstr):
        """__init__.

        :Parameters:
          errno : int
            Error code.
          errstr : unicode
            Error description.
        """
        Exception.__init__(self, int(errno), str(errstr))

    def __str__(self):
        return '[Errno %d] %s' % (self[0], self[1])



#!/usr/bin/env python
"""
Skype4Py distutils script.

Copyright (c) 2007-2009, Arkadiusz Wahlig

All rights reserved.

Distributed under the BSD License, see the
accompanying LICENSE file for more information.
"""

import os
import sys

from setuptools import setup

from distutils.cmd import Command
from distutils.command.install_lib import install_lib as old_install_lib

VERSION = '1.0.35'

# Change the current dir to where the setup.py is in case we're not there.
path = os.path.split(sys.argv[0])[0]
if path:
    os.chdir(path)


# So that the Skype4Py library may know that the setup is running.
sys.skype4py_setup = True


class install_lib(old_install_lib):
    """Handles the 'install_lib' command.

    This modified version of install_lib command installs only the necessary
    platform modules from the Skype4Py.api subpackage.
    """

    def install(self):
        # The build is done here, now we have to adapt it to current platform
        # before installing.
        self.adapt_build_to_platform()

        # Let the original method do the hard work of copying the files.
        outfiles = old_install_lib.install(self)

        # Also byte_compile for distribution usage.
        if outfiles is not None:
            self.byte_compile(outfiles)

        return outfiles

    def adapt_build_to_platform(self):
        # We have to remove unneeded files from the build directory. First,
        # decide what platform we're on; this code is similar to the one
        # in Skype4Py/api/__init__.py which decides what submodule to
        # import at runtime.
        if sys.platform[:3] == 'win':
            platform = 'windows'
        elif sys.platform == 'darwin':
            platform = 'darwin'
        else:
            platform = 'posix'

        # Scan the <build_dir>/Skype4Py/api directory and remove all files
        # which names do not start with either '__' (for __init__) or the
        # detected platform.
        path = os.path.join(self.build_dir, os.path.join('Skype4Py', 'api'))
        for name in os.listdir(path):
            if not (name.startswith('__') or name.startswith(platform)):
                os.remove(os.path.join(path, name))


class build_doc(Command):
    """Handles the 'build_doc' command.

    This command builds the documentation using epydoc. The documentation is then
    zipped using zipfile standard module.
    """

    description = 'build the documentation'
    user_options = [('pdf', None, 'Builds a PDF documentation instead of a HTML one.')]

    def initialize_options(self):
        self.pdf = None

    def finalize_options(self):
        pass

    def run(self):
        try:
            from epydoc import cli

            epydoc_config = os.path.join('doc', 'epydoc.conf')

            old_argv = sys.argv[1:]
            try:
                sys.argv[1:] = ['--config=%s' % epydoc_config]
                if self.pdf:
                    sys.argv.append('--pdf')
                    sys.argv.append('--output=doc/pdf/')
                else:
                    sys.argv.append('--html')
                    sys.argv.append('--output=doc/html/')

                cli.cli()
            finally:
                sys.argv[1:] = old_argv

            print 'zipping the documentation'
            import zipfile
            if self.pdf:
                doctype = 'pdf'
            else:
                doctype = 'html'
            name = 'Skype4Py-%s-%sdoc' % (VERSION, doctype)
            z = zipfile.ZipFile(os.path.join('doc', '%s.zip' % name),
                    'w', zipfile.ZIP_DEFLATED)
            path = os.path.join('doc', doctype)
            if self.pdf:
                z.write(os.path.join(path, 'api.pdf'), '%s.pdf' % name)
            else:
                for f in os.listdir(path):
                    z.write(os.path.join(path, f), os.path.join(name, f))
            z.close()

        except ImportError:
            print >>sys.stderr, 'epydoc not installed, skipping build_doc.'


commands = {'build_doc': build_doc,
            'install_lib': install_lib}

desc = open("README.rst").read() + "\n" + open("CHANGES.rst").read()

# start the distutils setup
setup(name='Skype4Py',
      version=VERSION,
      description='Skype API wrapper for Python.',
      long_description=desc,
      author='Arkadiusz Wahlig',
      author_email='arkadiusz.wahlig@gmail.com',
      maintainer='Mikko Ohtamaa',
      url='https://github.com/awahlig/skype4py',
      license='BSD License',
      platforms=['Windows', 'Linux', 'MacOS X'],
      packages=['Skype4Py', 'Skype4Py.api', 'Skype4Py.lang'],
      provides=['Skype4Py'],
      install_requires=['setuptools'],
      zip_safe=True,
      cmdclass=commands)



"""Utility functions and classes used internally by Skype4Py.
"""
__docformat__ = 'restructuredtext en'


import sys
import weakref
import threading
import logging
from new import instancemethod


__all__ = ['tounicode', 'path2unicode', 'unicode2path', 'chop', 'args2dict', 'quote',
           'split', 'cndexp', 'EventHandlingBase', 'Cached', 'CachedCollection']


def tounicode(s):
    """Converts a string to a unicode string. Accepts two types or arguments. An UTF-8 encoded
    byte string or a unicode string (in the latter case, no conversion is performed).

    :Parameters:
      s : str or unicode
        String to convert to unicode.

    :return: A unicode string being the result of the conversion.
    :rtype: unicode
    """
    if isinstance(s, unicode):
        return s
    return str(s).decode('utf-8')
    
    
def path2unicode(path):
    """Decodes a file/directory path from the current file system encoding to unicode.

    :Parameters:
      path : str
        Encoded path.

    :return: Decoded path.
    :rtype: unicode
    """
    return path.decode(sys.getfilesystemencoding())
    

def unicode2path(path):
    """Encodes a file/directory path from unicode to the current file system encoding.

    :Parameters:
      path : unicode
        Decoded path.

    :return: Encoded path.
    :rtype: str
    """
    return path.encode(sys.getfilesystemencoding())


def chop(s, n=1, d=None):
    """Chops initial words from a string and returns a list of them and the rest of the string.
    The returned list is guaranteed to be n+1 long. If too little words are found in the string,
    a ValueError exception is raised.

    :Parameters:
      s : str or unicode
        String to chop from.
      n : int
        Number of words to chop.
      d : str or unicode
        Optional delimiter. Any white-char by default.

    :return: A list of n first words from the string followed by the rest of the string (``[w1, w2,
             ..., wn, rest_of_string]``).
    :rtype: list of: str or unicode
    """

    spl = s.split(d, n)
    if len(spl) == n:
        spl.append(s[:0])
    if len(spl) != n + 1:
        raise ValueError('chop: Could not chop %d words from \'%s\'' % (n, s))
    return spl


def args2dict(s):
    """Converts a string or comma-separated 'ARG="a value"' or 'ARG=value2' strings
    into a dictionary.

    :Parameters:
      s : str or unicode
        Input string.

    :return: ``{'ARG': 'value'}`` dictionary.
    :rtype: dict
    """

    d = {}
    while s:
        t, s = chop(s, 1, '=')
        if s.startswith('"'):
            # XXX: This function is used to parse strings from Skype. The question is,
            # how does Skype escape the double-quotes. The code below implements the
            # VisualBasic technique ("" -> ").
            i = 0
            while True:
                i = s.find('"', i+1)
                try:
                    if s[i+1] != '"':
                        break
                    else:
                        i += 1
                except IndexError:
                    break
            if i > 0:
                d[t] = s[1:i].replace('""', '"')
                if s[i+1:i+3] == ', ':
                    i += 2
                s = s[i+1:]
            else:
                d[t] = s
                break
        else:
            i = s.find(', ')
            if i >= 0:
                d[t] = s[:i]
                s = s[i+2:]
            else:
                d[t] = s
                break
    return d


def quote(s, always=False):
    """Adds double-quotes to string if it contains spaces.

    :Parameters:
      s : str or unicode
        String to add double-quotes to.
      always : bool
        If True, adds quotes even if the input string contains no spaces.

    :return: If the given string contains spaces or <always> is True, returns the string enclosed in
             double-quotes. Otherwise returns the string unchanged.
    :rtype: str or unicode
    """

    if always or ' ' in s:
        return '"%s"' % s.replace('"', '""') # VisualBasic double-quote escaping.
    return s


def split(s, d=None):
    """Splits a string.

    :Parameters:
      s : str or unicode
        String to split.
      d : str or unicode
        Optional delimiter. Any white-char by default.

    :return: A list of words or ``[]`` if the string was empty.
    :rtype: list of str or unicode

    :note: This function works like ``s.split(d)`` except that it always returns an empty list
           instead of ``['']`` for empty strings.
    """

    if s:
        return s.split(d)
    return []


def cndexp(condition, truevalue, falsevalue):
    """Simulates a conditional expression known from C or Python 2.5.

    :Parameters:
      condition : any
        Tells what should be returned.
      truevalue : any
        Value returned if condition evaluates to True.
      falsevalue : any
        Value returned if condition evaluates to False.

    :return: Either truevalue or falsevalue depending on condition.
    :rtype: same as type of truevalue or falsevalue
    """

    if condition:
        return truevalue
    return falsevalue


class EventSchedulerThread(threading.Thread):
    def __init__(self, name, after, handlers, args, kwargs):
        """Initializes the object.
        
        :Parameters:
          name : str
            Event name.
          after : threading.Thread or None
            If not None, a thread that needs to end before this
            one starts.
          handlers : iterable
            Iterable of callable event handlers.
          args : tuple
            Positional arguments for the event handlers.
          kwargs : dict
            Keyword arguments for the event handlers.

        :note: When the thread is started (using the ``start`` method), it iterates over
               the handlers and calls them with the supplied arguments.
        """
        threading.Thread.__init__(self, name='Skype4Py %s event scheduler' % name)
        self.setDaemon(False)
        self.after = after
        self.handlers = handlers
        self.args = args
        self.kwargs = kwargs

    def run(self):
        if self.after:
            self.after.join()
            self.after = None # Remove the reference.
        for handler in self.handlers:
            handler(*self.args, **self.kwargs)


class EventHandlingBase(object):
    """This class is used as a base by all classes implementing event handlers.

    Look at known subclasses (above in epydoc) to see which classes will allow you to
    attach your own callables (event handlers) to certain events occurring in them.

    Read the respective classes documentations to learn what events are provided by them. The
    events are always defined in a class whose name consist of the name of the class it provides
    events for followed by ``Events``). For example class `Skype` provides events defined in
    `SkypeEvents`. The events class is always defined in the same module as the main class.

    The events class tells you what events you can assign your event handlers to, when do they
    occur and what arguments should your event handlers accept.

    There are three ways of attaching an event handler to an event.

    ``Events`` object
    =================

       Write your event handlers as methods of a class. The superclass of your class
       is not important for Skype4Py, it will just look for methods with appropriate names.
       The names of the methods and their arguments lists can be found in respective events
       classes (see above).

       Pass an instance of this class as the ``Events`` argument to the constructor of
       a class whose events you are interested in. For example:

       .. python::

           import Skype4Py

           class MySkypeEvents:
               def UserStatus(self, Status):
                   print 'The status of the user changed'

           skype = Skype4Py.Skype(Events=MySkypeEvents())
           
       If your application is build around a class, you may want to use is for Skype4Py
       events too. For example:
       
       .. python::
       
           import Skype4Py
           
           class MyApplication:
               def __init__(self):
                   self.skype = Skype4Py.Skype(Events=self)
                   
               def UserStatus(self, Status):
                   print 'The status of the user changed'
                   
       This lets you access the `Skype` object (``self.skype``) without using global
       variables.

       In both examples, the ``UserStatus`` method will be called when the status of the
       user currently logged into Skype is changed.

    ``On...`` properties
    ====================

       This method lets you use any callables as event handlers. Simply assign them to ``On...``
       properties (where "``...``" is the name of the event) of the object whose events you are
       interested in. For example:
       
       .. python::

           import Skype4Py

           def user_status(Status):
               print 'The status of the user changed'

           skype = Skype4Py.Skype()
           skype.OnUserStatus = user_status

       The ``user_status`` function will be called when the status of the user currently logged
       into Skype is changed.

       The names of the events and their arguments lists can be found in respective events
       classes (see above). Note that there is no ``self`` argument (which can be seen in the events
       classes) simply because our event handler is a function, not a method.

    ``RegisterEventHandler`` / ``UnregisterEventHandler`` methods
    =============================================================

       This method, like the second one, also lets you use any callables as event handlers. However,
       it also lets you assign many event handlers to a single event. This may be useful if for
       example you need to momentarily attach an event handler without disturbing other parts of
       your code already using one of the above two methods.

       In this case, you use `RegisterEventHandler` and `UnregisterEventHandler` methods
       of the object whose events you are interested in. For example:
       
       .. python::

           import Skype4Py

           def user_status(Status):
               print 'The status of the user changed'

           skype = Skype4Py.Skype()
           skype.RegisterEventHandler('UserStatus', user_status)

       The ``user_status`` function will be called when the status of the user currently logged
       into Skype is changed.

       The names of the events and their arguments lists should be taken from respective events
       classes (see above). Note that there is no ``self`` argument (which can be seen in the events
       classes) simply because our event handler is a function, not a method.
       
       All handlers attached to a single event will be called serially in the order they were
       registered.

    Multithreading warning
    ======================

       All event handlers are called on separate threads, never on the main one. At any given time,
       there is at most one thread per event calling your handlers. This means that when many events
       of the same type occur at once, the handlers will be called one after another. Different events
       will be handled simultaneously.
    
    Cyclic references note
    ======================

       Prior to Skype4Py 1.0.32.0, the library used weak references to the handlers. This was removed
       to avoid confusion and simplify/speed up the code. If cyclic references do occur, they are
       expected to be removed by the Python's garbage collector which should always be present as
       the library is expected to work in a relatively resource rich environment which is needed
       by the Skype client anyway.
    """
    # Initialized by the _AddEvents() class method.
    _EventNames = []

    def __init__(self):
        """Initializes the object.
        """
        # Event -> EventSchedulerThread object mapping. Use WeakValueDictionary to let the
        # threads be freed after they are finished.
        self._EventThreads = weakref.WeakValueDictionary()
        self._EventHandlerObject = None # Current "Events" object.
        self._DefaultEventHandlers = {} # "On..." handlers.
        self._EventHandlers = {} # "RegisterEventHandler" handlers.
        self.__Logger = logging.getLogger('Skype4Py.utils.EventHandlingBase')

        # Initialize the _EventHandlers mapping.
        for event in self._EventNames:
            self._EventHandlers[event] = []

    def _CallEventHandler(self, Event, *Args, **KwArgs):
        """Calls all event handlers defined for given Event, additional parameters
        will be passed unchanged to event handlers, all event handlers are fired on
        separate threads.
        
        :Parameters:
          Event : str
            Name of the event.
          Args
            Positional arguments for the event handlers.
          KwArgs
            Keyword arguments for the event handlers.
        """
        if Event not in self._EventHandlers:
            raise ValueError('%s is not a valid %s event name' % (Event, self.__class__.__name__))
        args = map(repr, Args) + ['%s=%s' % (key, repr(value)) for key, value in KwArgs.items()]
        self.__Logger.debug('calling %s: %s', Event, ', '.join(args))
        # Get a list of handlers for this event.
        try:
            handlers = [self._DefaultEventHandlers[Event]]
        except KeyError:
            handlers = []
        try:
            handlers.append(getattr(self._EventHandlerObject, Event))
        except AttributeError:
            pass
        handlers.extend(self._EventHandlers[Event])
        # Proceed only if there are handlers.
        if handlers:
            # Get the last thread for this event.
            after = self._EventThreads.get(Event, None)
            # Create a new thread, pass the last one to it so it can wait until it is finished.
            thread = EventSchedulerThread(Event, after, handlers, Args, KwArgs)
            # Store a weak reference to the new thread for this event.
            self._EventThreads[Event] = thread
            # Start the thread.
            thread.start()

    def RegisterEventHandler(self, Event, Target):
        """Registers any callable as an event handler.

        :Parameters:
          Event : str
            Name of the event. For event names, see the respective ``...Events`` class.
          Target : callable
            Callable to register as the event handler.

        :return: True is callable was successfully registered, False if it was already registered.
        :rtype: bool

        :see: `UnregisterEventHandler`
        """
        if not callable(Target):
            raise TypeError('%s is not callable' % repr(Target))
        if Event not in self._EventHandlers:
            raise ValueError('%s is not a valid %s event name' % (Event, self.__class__.__name__))
        if Target in self._EventHandlers[Event]:
            return False
        self._EventHandlers[Event].append(Target)
        self.__Logger.info('registered %s: %s', Event, repr(Target))
        return True

    def UnregisterEventHandler(self, Event, Target):
        """Unregisters an event handler previously registered with `RegisterEventHandler`.

        :Parameters:
          Event : str
            Name of the event. For event names, see the respective ``...Events`` class.
          Target : callable
            Callable to unregister.

        :return: True if callable was successfully unregistered, False if it wasn't registered
                 first.
        :rtype: bool

        :see: `RegisterEventHandler`
        """
        if not callable(Target):
            raise TypeError('%s is not callable' % repr(Target))
        if Event not in self._EventHandlers:
            raise ValueError('%s is not a valid %s event name' % (Event, self.__class__.__name__))
        if Target in self._EventHandlers[Event]:
            self._EventHandlers[Event].remove(Target)
            self.__Logger.info('unregistered %s: %s', Event, repr(Target))
            return True
        return False

    def _SetDefaultEventHandler(self, Event, Target):
        if Target:
            if not callable(Target):
                raise TypeError('%s is not callable' % repr(Target))
            self._DefaultEventHandlers[Event] = Target
            self.__Logger.info('set default %s: %s', Event, repr(Target))
        else:
            try:
                del self._DefaultEventHandlers[Event]
            except KeyError:
                pass

    def _GetDefaultEventHandler(self, Event):
        try:
            return self._DefaultEventHandlers[Event]
        except KeyError:
            return None

    def _SetEventHandlerObject(self, Object):
        """Registers an object as events handler, object should contain methods with names
        corresponding to event names, only one object may be registered at a time.
        
        :Parameters:
          Object
            Object to register. May be None in which case the currently registered object
            will be unregistered.
        """
        self._EventHandlerObject = Object
        self.__Logger.info('set object: %s', repr(Object))

    @classmethod
    def _AddEvents(cls, Class):
        """Adds events based on the attributes of the given ``...Events`` class.
        
        :Parameters:
          Class : class
            An `...Events` class whose methods define events that may occur in the
            instances of the current class.
        """
        def make_event(event):
            return property(lambda self: self._GetDefaultEventHandler(event),
                             lambda self, Value: self._SetDefaultEventHandler(event, Value))
        for event in dir(Class):
            if not event.startswith('_'):
                setattr(cls, 'On%s' % event, make_event(event))
                cls._EventNames.append(event)


class Cached(object):
    """Base class for all cached objects.

    Every object has an owning object a handle. Owning object is where the cache is
    maintained, handle identifies an object of given type.

    Thanks to the caching, trying to create two objects with the same owner and handle
    yields exactly the same object. The cache itself is based on weak references so
    not referenced objects are automatically removed from the cache.

    Because the ``__init__`` method will be called no matter if the object already
    existed or not, it is recommended to use the `_Init` method instead.
    """
    # Subclasses have to define a type/classmethod/staticmethod called
    # _ValidateHandle(Handle)
    # which is called by classmethod__new__ to validate the handle passed to
    # it before it is stored in the instance.

    def __new__(cls, Owner, Handle):
        Handle = cls._ValidateHandle(Handle)
        key = (cls, Handle)
        try:
            return Owner._ObjectCache[key]
        except KeyError:
            obj = object.__new__(cls)
            Owner._ObjectCache[key] = obj
            obj._Owner = Owner
            obj._Handle = Handle
            obj._Init()
            return obj
        except AttributeError:
            raise TypeError('%s is not a cached objects owner' % repr(Owner))
            
    def _Init(self):
        """Initializes the cached object. Receives all the arguments passed to the
        constructor The default implementation stores the ``Owner`` in
        ``self._Owner`` and ``Handle`` in ``self._Handle``.
        
        This method should be used instead of ``__init__`` to prevent double
        initialization.
        """

    def __copy__(self):
        return self
        
    def __repr__(self, *Attrs):
        if not Attrs:
            Attrs = ['_Handle']
        return '<%s.%s with %s>' % (self.__class__.__module__, self.__class__.__name__,
            ', '.join('%s=%s' % (name, repr(getattr(self, name))) for name in Attrs))
        
    def _MakeOwner(self):
        """Prepares the object for use as an owner for other cached objects.
        """
        self._CreateOwner(self)

    @staticmethod
    def _CreateOwner(Object):
        """Prepares any object for use as an owner for cached objects.
        
        :Parameters:
          Object
            Object that should be turned into a cached objects owner.
        """
        Object._ObjectCache = weakref.WeakValueDictionary()


class CachedCollection(object):
    """
    """
    _CachedType = Cached
    
    def __init__(self, Owner, Handles=[], Items=[]):
        self._Owner = Owner
        self._Handles = map(self._CachedType._ValidateHandle, Handles)
        for item in Items:
            self.append(item)

    def _AssertItem(self, Item):
        if not isinstance(Item, self._CachedType):
            raise TypeError('expected %s instance' % repr(self._CachedType))
        if self._Owner is not Item._Owner:
            raise TypeError('expected %s owned item' % repr(self._Owner))
        
    def _AssertCollection(self, Col):
        if not isinstance(Col, self.__class__):
            raise TypeError('expected %s instance' % repr(self.__class__))
        if self._CachedType is not Col._CachedType:
            raise TypeError('expected collection of %s' % repr(self._CachedType))
        if self._Owner is not Col._Owner:
            raise TypeError('expected %s owned collection' % repr(self._Owner))

    def __len__(self):
        return len(self._Handles)

    def __getitem__(self, Key):
        if isinstance(Key, slice):
            return self.__class__(self._Owner, self._Handles[Key])
        return self._CachedType(self._Owner, self._Handles[Key])

    def __setitem__(self, Key, Item):
        if isinstance(Key, slice):
            handles = []
            for it in Item:
                self._AssertItem(it)
                handles.append(it._Handle)
            self._Handlers[Key] = handles
        else:
            self._AssertItem(Item)
            self._Handles[Key] = Item._Handle

    def __delitem__(self, Key):
        del self._Handles[Key]

    def __iter__(self):
        for handle in self._Handles:
            yield self._CachedType(self._Owner, handle)

    def __contains__(self, Item):
        try:
            self._AssertItem(Item)
        except TypeError:
            return False
        return (Item._Handle in self._Handles)

    def __add__(self, Other):
        self._AssertCollection(Other)
        return self.__class__(self._Owner, self._Handles +
                          Other._Handles)

    def __iadd__(self, Other):
        self._AssertCollection(Other)
        self._Handles += Other._Handles
        return self

    def __mul__(self, Times):
        return self.__class__(self._Owner, self._Handles * Times)
    __rmul__ = __mul__

    def __imul__(self, Times):
        self._Handles *= Times
        return self
        
    def __copy__(self):
        obj = self.__class__(self._Owner)
        obj._Handles = self._Handles[:]
        return obj

    def append(self, item):
        """
        """
        self._AssertItem(item)
        self._Handles.append(item._Handle)

    def count(self, item):
        """
        """
        self._AssertItem(item)
        return self._Handles.count(item._Handle)

    def index(self, item):
        """
        """
        self._AssertItem(item)
        return self._Handles.index(item._Handle)

    def extend(self, seq):
        """
        """
        self.__iadd__(seq)

    def insert(self, index, item):
        """
        """
        self._AssertItem(item)
        self._Handles.insert(index, item._Handle)

    def pop(self, pos=-1):
        """
        """
        return self._CachedType(self._Owner, self._Handles.pop(pos))

    def remove(self, item):
        """
        """
        self._AssertItem(item)
        self._Handles.remove(item._Handle)

    def reverse(self):
        """
        """
        self._Handles.reverse()

    def sort(self, cmp=None, key=None, reverse=False):
        """
        """
        if key is None:
            wrapper = lambda x: self._CachedType(self._Owner, x)
        else:
            wrapper = lambda x: key(self._CachedType(self._Owner, x))
        self._Handles.sort(cmp, wrapper, reverse)

    def Add(self, Item):
        """
        """
        self.append(Item)

    def Remove(self, Index):
        """
        """
        del self[Index]

    def RemoveAll(self):
        """
        """
        del self[:]

    def Item(self, Index):
        """
        """
        return self[Index]

    def _GetCount(self):
        return len(self)

    Count = property(_GetCount,
    doc="""
    """)



"""Short messaging system.
"""
__docformat__ = 'restructuredtext en'


from utils import *


class SmsMessage(Cached):
    """Represents an SMS message.
    """
    _ValidateHandle = int

    def __repr__(self):
        return Cached.__repr__(self, 'Id')

    def _Alter(self, AlterName, Args=None):
        return self._Owner._Alter('SMS', self.Id, AlterName, Args)

    def _Init(self):
        self._MakeOwner()

    def _Property(self, PropName, Set=None, Cache=True):
        return self._Owner._Property('SMS', self.Id, PropName, Set, Cache)

    def Delete(self):
        """Deletes this SMS message.
        """
        self._Owner._DoCommand('DELETE SMS %s' % self.Id)

    def MarkAsSeen(self):
        """Marks this SMS message as seen.
        """
        self._Owner._DoCommand('SET SMS %s SEEN' % self.Id)
 
    def Send(self):
        """Sends this SMS message.
        """
        self._Alter('SEND')

    def _GetBody(self):
        return self._Property('BODY')

    def _SetBody(self, Value):
        self._Property('BODY', Value)

    Body = property(_GetBody, _SetBody,
    doc="""Text of this SMS message.

    :type: unicode
    """)

    def _GetChunks(self):
        return SmsChunkCollection(self, xrange(int(chop(self._Property('CHUNKING', Cache=False))[0])))

    Chunks = property(_GetChunks,
    doc="""Chunks of this SMS message. More than one if this is a multi-part message.

    :type: `SmsChunkCollection`
    """)

    def _GetDatetime(self):
        from datetime import datetime
        return datetime.fromtimestamp(self.Timestamp)

    Datetime = property(_GetDatetime,
    doc="""Timestamp of this SMS message as datetime object.

    :type: datetime.datetime
    """)

    def _GetFailureReason(self):
        return str(self._Property('FAILUREREASON'))

    FailureReason = property(_GetFailureReason,
    doc="""Reason an SMS message failed. Read this if `Status` == `enums.smsMessageStatusFailed`.

    :type: `enums`.smsFailureReason*
    """)

    def _GetId(self):
        return self._Handle

    Id = property(_GetId,
    doc="""Unique SMS message Id.

    :type: int
    """)

    def _GetIsFailedUnseen(self):
        return (self._Property('IS_FAILED_UNSEEN') == 'TRUE')

    IsFailedUnseen = property(_GetIsFailedUnseen,
    doc="""Tells if a failed SMS message was unseen.

    :type: bool
    """)

    def _GetPrice(self):
        return int(self._Property('PRICE'))

    Price = property(_GetPrice,
    doc="""SMS price. Expressed using `PricePrecision`. For a value expressed using `PriceCurrency`, use `PriceValue`.

    :type: int

    :see: `PriceCurrency`, `PricePrecision`, `PriceToText`, `PriceValue`
    """)

    def _GetPriceCurrency(self):
        return self._Property('PRICE_CURRENCY')

    PriceCurrency = property(_GetPriceCurrency,
    doc="""SMS price currency.

    :type: unicode

    :see: `Price`, `PricePrecision`, `PriceToText`, `PriceValue`
    """)

    def _GetPricePrecision(self):
        return int(self._Property('PRICE_PRECISION'))

    PricePrecision = property(_GetPricePrecision,
    doc="""SMS price precision.

    :type: int

    :see: `Price`, `PriceCurrency`, `PriceToText`, `PriceValue`
    """)

    def _GetPriceToText(self):
        return (u'%s %.3f' % (self.PriceCurrency, self.PriceValue)).strip()

    PriceToText = property(_GetPriceToText,
    doc="""SMS price as properly formatted text with currency.

    :type: unicode

    :see: `Price`, `PriceCurrency`, `PricePrecision`, `PriceValue`
    """)

    def _GetPriceValue(self):
        if self.Price < 0:
            return 0.0
        return float(self.Price) / (10 ** self.PricePrecision)

    PriceValue = property(_GetPriceValue,
    doc="""SMS price. Expressed in `PriceCurrency`.

    :type: float

    :see: `Price`, `PriceCurrency`, `PricePrecision`, `PriceToText`
    """)

    def _GetReplyToNumber(self):
        return str(self._Property('REPLY_TO_NUMBER'))

    def _SetReplyToNumber(self, Value):
        self._Property('REPLY_TO_NUMBER', Value)

    ReplyToNumber = property(_GetReplyToNumber, _SetReplyToNumber,
    doc="""Reply-to number for this SMS message.

    :type: str
    """)

    def _SetSeen(self, Value):
        from warnings import warn
        warn('SmsMessage.Seen = x: Use SmsMessage.MarkAsSeen() instead.', DeprecationWarning, stacklevel=2)
        if Value:
            self.MarkAsSeen()
        else:
            raise SkypeError(0, 'Seen can only be set to True')

    Seen = property(fset=_SetSeen,
    doc="""Set the read status of the SMS message. Accepts only True value.

    :type: bool

    :deprecated: Extremely unpythonic, use `MarkAsSeen` instead.
    """)

    def _GetStatus(self):
        return str(self._Property('STATUS'))

    Status = property(_GetStatus,
    doc="""SMS message status.

    :type: `enums`.smsMessageStatus*
    """)

    def _GetTargetNumbers(self):
        return tuple(split(self._Property('TARGET_NUMBERS'), ', '))

    def _SetTargetNumbers(self, Value):
        self._Property('TARGET_NUMBERS', ', '.join(Value))

    TargetNumbers = property(_GetTargetNumbers, _SetTargetNumbers,
    doc="""Target phone numbers.

    :type: tuple of str
    """)

    def _GetTargets(self):
        return SmsTargetCollection(self, split(self._Property('TARGET_NUMBERS'), ', '))

    Targets = property(_GetTargets,
    doc="""Target objects.

    :type: `SmsTargetCollection`
    """)

    def _GetTimestamp(self):
        return float(self._Property('TIMESTAMP'))

    Timestamp = property(_GetTimestamp,
    doc="""Timestamp of this SMS message.

    :type: float

    :see: `Datetime`
    """)

    def _GetType(self):
        return str(self._Property('TYPE'))

    Type = property(_GetType,
    doc="""SMS message type

    :type: `enums`.smsMessageType*
    """)


class SmsMessageCollection(CachedCollection):
    _CachedType = SmsMessage


class SmsChunk(Cached):
    """Represents a single chunk of a multi-part SMS message.
    """
    _ValidateHandle = int

    def __repr__(self):
        return Cached.__repr__(self, 'Id', 'Message')

    def _GetCharactersLeft(self):
        count, left = map(int, chop(self.Message._Property('CHUNKING', Cache=False)))
        if self.Id == count - 1:
            return left
        return 0

    CharactersLeft = property(_GetCharactersLeft,
    doc="""CharactersLeft.

    :type: int
    """)

    def _GetId(self):
        return self._Handle

    Id = property(_GetId,
    doc="""SMS chunk Id.

    :type: int
    """)

    def _GetMessage(self):
        return self._Owner

    Message = property(_GetMessage,
    doc="""SMS message associated with this chunk.

    :type: `SmsMessage`
    """)

    def _GetText(self):
        return self.Message._Property('CHUNK %s' % self.Id)

    Text = property(_GetText,
    doc="""Text (body) of this SMS chunk.

    :type: unicode
    """)


class SmsChunkCollection(CachedCollection):
    _CachedType = SmsChunk


class SmsTarget(Cached):
    """Represents a single target of a multi-target SMS message.
    """
    _ValidateHandle = str

    def __repr__(self):
        return Cached.__repr__(self, 'Number', 'Message')

    def _GetMessage(self):
        return self._Owner

    Message = property(_GetMessage,
    doc="""An SMS message object this target refers to.

    :type: `SmsMessage`
    """)

    def _GetNumber(self):
        return self._Handle

    Number = property(_GetNumber,
    doc="""Target phone number.

    :type: str
    """)

    def _GetStatus(self):
        for t in split(self.Message._Property('TARGET_STATUSES'), ', '):
            number, status = t.split('=')
            if number == self.Number:
                return str(status)

    Status = property(_GetStatus,
    doc="""Status of this target.

    :type: `enums`.smsTargetStatus*
    """)


class SmsTargetCollection(CachedCollection):
    _CachedType = SmsTarget



apiAttachAvailable = u'API disponible'
apiAttachNotAvailable = u'Indisponible'
apiAttachPendingAuthorization = u'Autorisation en attente'
apiAttachRefused = u'Refus\xe9'
apiAttachSuccess = u'Connexion r\xe9ussie'
apiAttachUnknown = u'Inconnu'
budDeletedFriend = u'Supprim\xe9 de la liste d\u2019amis'
budFriend = u'Ami'
budNeverBeenFriend = u"N'a jamais \xe9t\xe9 ajout\xe9 \xe0 la liste d\u2019amis"
budPendingAuthorization = u'Autorisation en attente'
budUnknown = u'Inconnu'
cfrBlockedByRecipient = u'Appel bloqu\xe9 par le destinataire'
cfrMiscError = u'Erreurs diverses'
cfrNoCommonCodec = u'Aucun codec en commun'
cfrNoProxyFound = u'Aucun proxy trouv\xe9'
cfrNotAuthorizedByRecipient = u'Utilisateur actuel non autoris\xe9 par le destinataire'
cfrRecipientNotFriend = u'Destinataire n\u2019est pas un ami'
cfrRemoteDeviceError = u'Erreur E/S audio distante'
cfrSessionTerminated = u'Session termin\xe9e'
cfrSoundIOError = u'Erreur E/S son'
cfrSoundRecordingError = u'Erreur d\u2019enregistrement du son'
cfrUnknown = u'Inconnu'
cfrUserDoesNotExist = u'Utilisateur/n\xb0 de t\xe9l\xe9phone inexistant'
cfrUserIsOffline = u'Il/Elle est D\xe9connect\xe9(e)'
chsAllCalls = u'Ancien dialogue'
chsDialog = u'Dialogue'
chsIncomingCalls = u'Attente multi acceptation'
chsLegacyDialog = u'Ancien dialogue'
chsMissedCalls = u'Dialogue'
chsMultiNeedAccept = u'Attente multi acceptation'
chsMultiSubscribed = u'Multi abonn\xe9s'
chsOutgoingCalls = u'Multi abonn\xe9s'
chsUnknown = u'Inconnu'
chsUnsubscribed = u'D\xe9sabonn\xe9'
clsBusy = u'Occup\xe9'
clsCancelled = u'Annul\xe9'
clsEarlyMedia = u'Lecture flux m\xe9dia (Early Media)'
clsFailed = u"D\xe9sol\xe9, l'appel a \xe9chou\xe9 !"
clsFinished = u'Termin\xe9'
clsInProgress = u'Appel en cours...'
clsLocalHold = u'En attente locale'
clsMissed = u'Appel en absence'
clsOnHold = u'En attente'
clsRefused = u'Refus\xe9'
clsRemoteHold = u'En attente \xe0 distance'
clsRinging = u'un appel'
clsRouting = u'Routage'
clsTransferred = u'Inconnu'
clsTransferring = u'Inconnu'
clsUnknown = u'Inconnu'
clsUnplaced = u'Jamais plac\xe9'
clsVoicemailBufferingGreeting = u'Buff\xe9risation du message d\u2019accueil'
clsVoicemailCancelled = u'Message vocal annul\xe9'
clsVoicemailFailed = u'Echec du message vocal'
clsVoicemailPlayingGreeting = u'Lecture du message d\u2019accueil'
clsVoicemailRecording = u'Enregistrement sur la boite vocale'
clsVoicemailSent = u'Message vocal envoy\xe9'
clsVoicemailUploading = u'T\xe9l\xe9chargement du message vocal'
cltIncomingP2P = u'Appel P2P entrant'
cltIncomingPSTN = u'Appel entrant'
cltOutgoingP2P = u'Appel P2P sortant'
cltOutgoingPSTN = u'Appel sortant'
cltUnknown = u'Inconnu'
cmeAddedMembers = u'A ajout\xe9 des membres'
cmeCreatedChatWith = u'Cr\xe9\xe9 un dialogue avec'
cmeEmoted = u'Inconnu'
cmeLeft = u'Laiss\xe9'
cmeSaid = u'A dit'
cmeSawMembers = u'A vu des membres'
cmeSetTopic = u'A d\xe9fini un sujet'
cmeUnknown = u'Inconnu'
cmsRead = u'Lu'
cmsReceived = u'Re\xe7u'
cmsSending = u'Envoi en cours...'
cmsSent = u'Envoy\xe9'
cmsUnknown = u'Inconnu'
conConnecting = u'Connexion en cours'
conOffline = u'D\xe9connect\xe9'
conOnline = u'Connect\xe9'
conPausing = u'En pause'
conUnknown = u'Inconnu'
cusAway = u'Absent'
cusDoNotDisturb = u'Ne pas d\xe9ranger'
cusInvisible = u'Invisible'
cusLoggedOut = u'D\xe9connect\xe9'
cusNotAvailable = u'Indisponible'
cusOffline = u'D\xe9connect\xe9'
cusOnline = u'Connect\xe9'
cusSkypeMe = u'Accessible'
cusUnknown = u'Inconnu'
cvsBothEnabled = u'Envoi et r\xe9ception vid\xe9o'
cvsNone = u'Pas de vid\xe9o'
cvsReceiveEnabled = u'R\xe9ception vid\xe9o'
cvsSendEnabled = u'Envoi vid\xe9o'
cvsUnknown = u''
grpAllFriends = u'Tous les amis'
grpAllUsers = u'Tous les utilisateurs'
grpCustomGroup = u'Personnalis\xe9'
grpOnlineFriends = u'Amis en ligne'
grpPendingAuthorizationFriends = u'Autorisation en attente'
grpProposedSharedGroup = u'Proposed Shared Group'
grpRecentlyContactedUsers = u'Utilisateurs r\xe9cemment contact\xe9s'
grpSharedGroup = u'Shared Group'
grpSkypeFriends = u'Amis Skype'
grpSkypeOutFriends = u'Amis SkypeOut'
grpUngroupedFriends = u'Amis sans groupe'
grpUnknown = u'Inconnu'
grpUsersAuthorizedByMe = u'Autoris\xe9 par moi'
grpUsersBlockedByMe = u'Bloqu\xe9 par moi'
grpUsersWaitingMyAuthorization = u'En attente de mon autorisation'
leaAddDeclined = u'Ajout refus\xe9'
leaAddedNotAuthorized = u'La personne ajout\xe9e doit \xeatre autoris\xe9e'
leaAdderNotFriend = u'La personne qui ajoute doit \xeatre un ami'
leaUnknown = u'Inconnu'
leaUnsubscribe = u'D\xe9sabonn\xe9'
leaUserIncapable = u'Utilisateur incapable'
leaUserNotFound = u'Utilisateur introuvable'
olsAway = u'Absent'
olsDoNotDisturb = u'Ne pas d\xe9ranger'
olsNotAvailable = u'Indisponible'
olsOffline = u'D\xe9connect\xe9'
olsOnline = u'Connect\xe9'
olsSkypeMe = u'Accessible'
olsSkypeOut = u'SkypeOut'
olsUnknown = u'Inconnu'
smsMessageStatusComposing = u'Composing'
smsMessageStatusDelivered = u'Delivered'
smsMessageStatusFailed = u'Failed'
smsMessageStatusRead = u'Read'
smsMessageStatusReceived = u'Received'
smsMessageStatusSendingToServer = u'Sending to Server'
smsMessageStatusSentToServer = u'Sent to Server'
smsMessageStatusSomeTargetsFailed = u'Some Targets Failed'
smsMessageStatusUnknown = u'Unknown'
smsMessageTypeCCRequest = u'Confirmation Code Request'
smsMessageTypeCCSubmit = u'Confirmation Code Submit'
smsMessageTypeIncoming = u'Incoming'
smsMessageTypeOutgoing = u'Outgoing'
smsMessageTypeUnknown = u'Unknown'
smsTargetStatusAcceptable = u'Acceptable'
smsTargetStatusAnalyzing = u'Analyzing'
smsTargetStatusDeliveryFailed = u'Delivery Failed'
smsTargetStatusDeliveryPending = u'Delivery Pending'
smsTargetStatusDeliverySuccessful = u'Delivery Successful'
smsTargetStatusNotRoutable = u'Not Routable'
smsTargetStatusUndefined = u'Undefined'
smsTargetStatusUnknown = u'Unknown'
usexFemale = u'Femme'
usexMale = u'Homme'
usexUnknown = u'Inconnu'
vmrConnectError = u'Erreur de connexion'
vmrFileReadError = u'Erreur de lecture fichier'
vmrFileWriteError = u'Erreur d\u2019\xe9criture fichier'
vmrMiscError = u'Erreurs diverses'
vmrNoError = u'Pas d\u2019erreur'
vmrNoPrivilege = u'Pas de privil\xe8ge Voicemail'
vmrNoVoicemail = u'Aucun message vocal de ce type'
vmrPlaybackError = u'Erreur de lecture'
vmrRecordingError = u'Erreur d\u2019enregistrement'
vmrUnknown = u'Inconnu'
vmsBlank = u'Vierge'
vmsBuffering = u'Buff\xe9risation en cours'
vmsDeleting = u'Suppression en cours'
vmsDownloading = u'T\xe9l\xe9chargement en cours'
vmsFailed = u'\xc9chec'
vmsNotDownloaded = u'Non t\xe9l\xe9charg\xe9'
vmsPlayed = u'Lu'
vmsPlaying = u'Lecture en cours'
vmsRecorded = u'Enregistr\xe9'
vmsRecording = u'Enregistrement sur la boite vocale'
vmsUnknown = u'Inconnu'
vmsUnplayed = u'Non lu'
vmsUploaded = u'T\xe9l\xe9charg\xe9'
vmsUploading = u'T\xe9l\xe9chargement en cours'
vmtCustomGreeting = u'Message d\u2019accueil personnalis\xe9'
vmtDefaultGreeting = u'Message d\u2019accueil par d\xe9faut'
vmtIncoming = u'R\xe9ception de message sur la boite vocale'
vmtOutgoing = u'Sortant'
vmtUnknown = u'Inconnu'
vssAvailable = u'Disponible'
vssNotAvailable = u'Indisponible'
vssPaused = u'En pause'
vssRejected = u'Rejet\xe9'
vssRunning = u'En cours'
vssStarting = u'D\xe9marrage'
vssStopping = u'En cours d\u2019arr\xeat'
vssUnknown = u'Inconnu'



"""Voicemails.
"""
__docformat__ = 'restructuredtext en'


from utils import *
from enums import *
from call import DeviceMixin


class Voicemail(Cached, DeviceMixin):
    """Represents a voicemail.
    """
    _ValidateHandle = int

    def __repr__(self):
        return Cached.__repr__(self, 'Id')

    def _Alter(self, AlterName, Args=None):
        return self._Owner._Alter('VOICEMAIL', self.Id, AlterName, Args)

    def _Property(self, PropName, Set=None, Cache=True):
        return self._Owner._Property('VOICEMAIL', self.Id, PropName, Set, Cache)

    def Delete(self):
        """Deletes this voicemail.
        """
        self._Alter('DELETE')

    def Download(self):
        """Downloads this voicemail object from the voicemail server to a local computer.
        """
        self._Alter('DOWNLOAD')

    def Open(self):
        """Opens and plays this voicemail.
        """
        self._Owner._DoCommand('OPEN VOICEMAIL %s' % self.Id)

    def SetUnplayed(self):
        """Changes the status of a voicemail from played to unplayed.
        """
        # Note. Due to a bug in Skype (tested using 3.8.0.115) the reply from
        # [ALTER VOICEMAIL <id> SETUNPLAYED] is [ALTER VOICEMAIL <id> DELETE]
        # causing the _Alter method to fail. Therefore we have to use a direct
        # _DoCommand instead. For the event of this being fixed, we don't
        # check for the "SETUNPLAYED"/"DELETE" part of the response.
        
        #self._Alter('SETUNPLAYED')
        self._Owner._DoCommand('ALTER VOICEMAIL %d SETUNPLAYED' % self.Id,
                               'ALTER VOICEMAIL %d' % self.Id)

    def StartPlayback(self):
        """Starts playing downloaded voicemail.
        """
        self._Alter('STARTPLAYBACK')

    def StartPlaybackInCall(self):
        """Starts playing downloaded voicemail during a call.
        """
        self._Alter('STARTPLAYBACKINCALL')

    def StartRecording(self):
        """Stops playing a voicemail greeting and starts recording a voicemail message.
        """
        self._Alter('STARTRECORDING')

    def StopPlayback(self):
        """Stops playing downloaded voicemail.
        """
        self._Alter('STOPPLAYBACK')

    def StopRecording(self):
        """Ends the recording of a voicemail message.
        """
        self._Alter('STOPRECORDING')

    def Upload(self):
        """Uploads recorded voicemail from a local computer to the voicemail server.
        """
        self._Alter('UPLOAD')

    def _GetAllowedDuration(self):
        return int(self._Property('ALLOWED_DURATION'))

    AllowedDuration = property(_GetAllowedDuration,
    doc="""Maximum voicemail duration in seconds allowed to leave to partner

    :type: int
    """)

    def _GetDatetime(self):
        from datetime import datetime
        return datetime.fromtimestamp(self.Timestamp)

    Datetime = property(_GetDatetime,
    doc="""Timestamp of this voicemail expressed using datetime.

    :type: datetime.datetime
    """)

    def _GetDuration(self):
        return int(self._Property('DURATION'))

    Duration = property(_GetDuration,
    doc="""Actual voicemail duration in seconds.

    :type: int
    """)

    def _GetFailureReason(self):
        return str(self._Property('FAILUREREASON'))

    FailureReason = property(_GetFailureReason,
    doc="""Voicemail failure reason. Read if `Status` == `enums.vmsFailed`.

    :type: `enums`.vmr*
    """)

    def _GetId(self):
        return self._Handle

    Id = property(_GetId,
    doc="""Unique voicemail Id.

    :type: int
    """)

    def _GetPartnerDisplayName(self):
        return self._Property('PARTNER_DISPNAME')

    PartnerDisplayName = property(_GetPartnerDisplayName,
    doc="""DisplayName for voicemail sender (for incoming) or recipient (for outgoing).

    :type: unicode
    """)

    def _GetPartnerHandle(self):
        return str(self._Property('PARTNER_HANDLE'))

    PartnerHandle = property(_GetPartnerHandle,
    doc="""Skypename for voicemail sender (for incoming) or recipient (for outgoing).

    :type: str
    """)

    def _GetStatus(self):
        return str(self._Property('STATUS'))

    Status = property(_GetStatus,
    doc="""Voicemail status.

    :type: `enums`.vms*
    """)

    def _GetTimestamp(self):
        return float(self._Property('TIMESTAMP'))

    Timestamp = property(_GetTimestamp,
    doc="""Timestamp of this voicemail.

    :type: float
    """)

    def _GetType(self):
        return str(self._Property('TYPE'))

    Type = property(_GetType,
    doc="""Voicemail type.

    :type: `enums`.vmt*
    """)


class VoicemailCollection(CachedCollection):
    _CachedType = Voicemail



apiAttachAvailable = u'API el\xe9rheto'
apiAttachNotAvailable = u'Nem el\xe9rheto'
apiAttachPendingAuthorization = u'F\xfcggoben l\xe9vo visszaigazol\xe1s'
apiAttachRefused = u'Elutas\xedtva'
apiAttachSuccess = u'Siker\xfclt'
apiAttachUnknown = u'Ismeretlen'
budDeletedFriend = u'T\xf6r\xf6lve a bar\xe1tok list\xe1j\xe1r\xf3l'
budFriend = u'Bar\xe1t'
budNeverBeenFriend = u'Soha nem volt a bar\xe1tok k\xf6z\xf6tt'
budPendingAuthorization = u'F\xfcggoben l\xe9vo visszaigazol\xe1s'
budUnknown = u'Ismeretlen'
cfrBlockedByRecipient = u'A h\xedv\xe1st blokkolta a c\xedmzett'
cfrMiscError = u'Egy\xe9b hiba'
cfrNoCommonCodec = u'Nincs szabv\xe1nyos kodek'
cfrNoProxyFound = u'Proxy nem tal\xe1lhat\xf3'
cfrNotAuthorizedByRecipient = u'A c\xedmzett nem igazolta vissza az aktu\xe1lis felhaszn\xe1l\xf3t'
cfrRecipientNotFriend = u'C\xedmzett nem tal\xe1lhat\xf3'
cfrRemoteDeviceError = u'Hanghiba a partnern\xe9l'
cfrSessionTerminated = u'V\xe9ge a h\xedv\xe1snak'
cfrSoundIOError = u'Hang I/O hiba'
cfrSoundRecordingError = u'Hangr\xf6gz\xedt\xe9si hiba'
cfrUnknown = u'Ismeretlen'
cfrUserDoesNotExist = u'A felhaszn\xe1l\xf3/telefonsz\xe1m nem l\xe9tezik'
cfrUserIsOffline = u'Kijelentkezett'
chsAllCalls = u'Kor\xe1bbi p\xe1rbesz\xe9d'
chsDialog = u'P\xe1rbesz\xe9d'
chsIncomingCalls = u'T\xf6bben akarj\xe1k fogadni'
chsLegacyDialog = u'Kor\xe1bbi p\xe1rbesz\xe9d'
chsMissedCalls = u'P\xe1rbesz\xe9d'
chsMultiNeedAccept = u'T\xf6bben akarj\xe1k fogadni'
chsMultiSubscribed = u'T\xf6bben feliratkoztak'
chsOutgoingCalls = u'T\xf6bben feliratkoztak'
chsUnknown = u'Ismeretlen'
chsUnsubscribed = u'Nincs feliratkozva'
clsBusy = u'Foglalt'
clsCancelled = u'Megszak\xedtva'
clsEarlyMedia = u'R\xe9gi zenesz\xe1m lej\xe1tsz\xe1sa'
clsFailed = u'A h\xedv\xe1s sikertelen.'
clsFinished = u'K\xe9sz'
clsInProgress = u'A besz\xe9lget\xe9s tart...'
clsLocalHold = u'Helyi h\xedv\xe1start\xe1s'
clsMissed = u'Nem fogadott h\xedv\xe1s tole:'
clsOnHold = u'H\xedv\xe1start\xe1s alatt'
clsRefused = u'Elutas\xedtva'
clsRemoteHold = u'T\xe1voli h\xedv\xe1start\xe1s'
clsRinging = u'h\xedv\xe1s'
clsRouting = u'\xdatvonal ir\xe1ny\xedt\xe1s'
clsTransferred = u'Ismeretlen'
clsTransferring = u'Ismeretlen'
clsUnknown = u'Ismeretlen'
clsUnplaced = u'Soha nem h\xedvta'
clsVoicemailBufferingGreeting = u'K\xf6sz\xf6nt\xe9s pufferel\xe9se'
clsVoicemailCancelled = u'Hang\xfczenet megszak\xedtva'
clsVoicemailFailed = u'Hang\xfczenet-hiba'
clsVoicemailPlayingGreeting = u'K\xf6sz\xf6nt\xe9s lej\xe1tsz\xe1sa'
clsVoicemailRecording = u'Hang\xfczenet felv\xe9tele t\xf6rt\xe9nik'
clsVoicemailSent = u'Hang\xfczenet elk\xfcldve'
clsVoicemailUploading = u'Hang\xfczenet felt\xf6lt\xe9se'
cltIncomingP2P = u'Bej\xf6vo k\xf6zvetlen h\xedv\xe1s'
cltIncomingPSTN = u'Bej\xf6vo telefonh\xedv\xe1s'
cltOutgoingP2P = u'Kimeno k\xf6zvetlen h\xedv\xe1s'
cltOutgoingPSTN = u'Kimeno telefonh\xedv\xe1s'
cltUnknown = u'Ismeretlen'
cmeAddedMembers = u'Hozz\xe1adott tagok'
cmeCreatedChatWith = u'Cseveg\xe9s l\xe9trehozva'
cmeEmoted = u'Ismeretlen'
cmeLeft = u'Elhagyta'
cmeSaid = u'Mondta'
cmeSawMembers = u'L\xe1tott tagok'
cmeSetTopic = u'T\xe9ma be\xe1ll\xedt\xe1sa'
cmeUnknown = u'Ismeretlen'
cmsRead = u'Elolvasva'
cmsReceived = u'Meg\xe9rkezett'
cmsSending = u'K\xfcld\xe9s...'
cmsSent = u'Elk\xfcldve'
cmsUnknown = u'Ismeretlen'
conConnecting = u'Kapcsol\xf3d\xe1s'
conOffline = u'Kijelentkezve'
conOnline = u'El\xe9rheto'
conPausing = u'Sz\xfcnetel'
conUnknown = u'Ismeretlen'
cusAway = u'Nincs a g\xe9pn\xe9l'
cusDoNotDisturb = u'Elfoglalt'
cusInvisible = u'L\xe1thatatlan'
cusLoggedOut = u'Kijelentkezve'
cusNotAvailable = u'Nem el\xe9rheto'
cusOffline = u'Kijelentkezve'
cusOnline = u'El\xe9rheto'
cusSkypeMe = u'H\xedv\xe1sk\xe9sz'
cusUnknown = u'Ismeretlen'
cvsBothEnabled = u'Vide\xf3 k\xfcld\xe9s \xe9s fogad\xe1s'
cvsNone = u'Nincs vide\xf3'
cvsReceiveEnabled = u'Vide\xf3 fogad\xe1s'
cvsSendEnabled = u'Vide\xf3 k\xfcld\xe9s'
cvsUnknown = u''
grpAllFriends = u'Minden bar\xe1t'
grpAllUsers = u'Minden felhaszn\xe1l\xf3'
grpCustomGroup = u'Egyedi'
grpOnlineFriends = u'El\xe9rheto bar\xe1tok'
grpPendingAuthorizationFriends = u'F\xfcggoben l\xe9vo visszaigazol\xe1s'
grpProposedSharedGroup = u'Proposed Shared Group'
grpRecentlyContactedUsers = u'Legut\xf3bb keresett felhaszn\xe1l\xf3k'
grpSharedGroup = u'Shared Group'
grpSkypeFriends = u'Skype bar\xe1tok'
grpSkypeOutFriends = u'SkypeOut bar\xe1tok'
grpUngroupedFriends = u'Nem csoportos\xedtott bar\xe1tok'
grpUnknown = u'Ismeretlen'
grpUsersAuthorizedByMe = u'Visszaigazoltak'
grpUsersBlockedByMe = u'Blokkoltak'
grpUsersWaitingMyAuthorization = u'Visszaigazol\xe1sra v\xe1rakoz\xf3k'
leaAddDeclined = u'Hozz\xe1ad\xe1s elutas\xedtva'
leaAddedNotAuthorized = u'A hozz\xe1adott szem\xe9lyt vissza kell igazolni'
leaAdderNotFriend = u'A hozz\xe1ad\xf3nak bar\xe1tnak kell lennie'
leaUnknown = u'Ismeretlen'
leaUnsubscribe = u'Nincs feliratkozva'
leaUserIncapable = u'Felhaszn\xe1l\xf3 nem tudja'
leaUserNotFound = u'Felhaszn\xe1l\xf3 nem tal\xe1lhat\xf3'
olsAway = u'Nincs a g\xe9pn\xe9l'
olsDoNotDisturb = u'Elfoglalt'
olsNotAvailable = u'Nem el\xe9rheto'
olsOffline = u'Kijelentkezve'
olsOnline = u'El\xe9rheto'
olsSkypeMe = u'H\xedv\xe1sk\xe9sz'
olsSkypeOut = u'SkypeOut (?????-????)'
olsUnknown = u'Ismeretlen'
smsMessageStatusComposing = u'Composing'
smsMessageStatusDelivered = u'Delivered'
smsMessageStatusFailed = u'Failed'
smsMessageStatusRead = u'Read'
smsMessageStatusReceived = u'Received'
smsMessageStatusSendingToServer = u'Sending to Server'
smsMessageStatusSentToServer = u'Sent to Server'
smsMessageStatusSomeTargetsFailed = u'Some Targets Failed'
smsMessageStatusUnknown = u'Unknown'
smsMessageTypeCCRequest = u'Confirmation Code Request'
smsMessageTypeCCSubmit = u'Confirmation Code Submit'
smsMessageTypeIncoming = u'Incoming'
smsMessageTypeOutgoing = u'Outgoing'
smsMessageTypeUnknown = u'Unknown'
smsTargetStatusAcceptable = u'Acceptable'
smsTargetStatusAnalyzing = u'Analyzing'
smsTargetStatusDeliveryFailed = u'Delivery Failed'
smsTargetStatusDeliveryPending = u'Delivery Pending'
smsTargetStatusDeliverySuccessful = u'Delivery Successful'
smsTargetStatusNotRoutable = u'Not Routable'
smsTargetStatusUndefined = u'Undefined'
smsTargetStatusUnknown = u'Unknown'
usexFemale = u'no'
usexMale = u'f\xe9rfi'
usexUnknown = u'Ismeretlen'
vmrConnectError = u'Csatlakoz\xe1si hiba'
vmrFileReadError = u'F\xe1jl olvas\xe1si hiba'
vmrFileWriteError = u'F\xe1jl \xedr\xe1si hiba'
vmrMiscError = u'Egy\xe9b hiba'
vmrNoError = u'Nincs hiba'
vmrNoPrivilege = u'Nincs hang\xfczenet jogosults\xe1ga'
vmrNoVoicemail = u'Nincs ilyen hang\xfczenet'
vmrPlaybackError = u'Visszaj\xe1tsz\xe1si hiba'
vmrRecordingError = u'Hib\xe1s felv\xe9tel'
vmrUnknown = u'Ismeretlen'
vmsBlank = u'\xdcres'
vmsBuffering = u'Pufferel\xe9s'
vmsDeleting = u'T\xf6rl\xe9s'
vmsDownloading = u'Let\xf6lt\xe9s'
vmsFailed = u'Nem siker\xfclt'
vmsNotDownloaded = u'Nincs let\xf6ltve'
vmsPlayed = u'Lej\xe1tszott'
vmsPlaying = u'Lej\xe1tsz\xe1s'
vmsRecorded = u'R\xf6gz\xedtett'
vmsRecording = u'Hang\xfczenet felv\xe9tele t\xf6rt\xe9nik'
vmsUnknown = u'Ismeretlen'
vmsUnplayed = u'Nem lej\xe1tszott'
vmsUploaded = u'Felt\xf6ltve'
vmsUploading = u'Felt\xf6lt\xe9s'
vmtCustomGreeting = u'Egy\xe9ni k\xf6sz\xf6nt\xe9s'
vmtDefaultGreeting = u'Alap\xe9rtelmezett k\xf6sz\xf6nt\xe9s'
vmtIncoming = u'Hang\xfczenet \xe9rkezik'
vmtOutgoing = u'Kimeno'
vmtUnknown = u'Ismeretlen'
vssAvailable = u'El\xe9rheto'
vssNotAvailable = u'Nem el\xe9rheto'
vssPaused = u'Sz\xfcnetel'
vssRejected = u'Visszautas\xedtva'
vssRunning = u'Fut'
vssStarting = u'Ind\xedt\xe1s'
vssStopping = u'Le\xe1ll\xedt\xe1s'
vssUnknown = u'Ismeretlen'



"""File transfers.
"""
__docformat__ = 'restructuredtext en'


import os

from utils import *


class FileTransfer(Cached):
    """Represents a file transfer.
    """
    _ValidateHandle = int

    def __repr__(self):
        return Cached.__repr__(self, 'Id')

    def _Alter(self, AlterName, Args=None):
        return self._Owner._Alter('FILETRANSFER', self.Id, AlterName, Args)

    def _Property(self, PropName, Set=None):
        return self._Owner._Property('FILETRANSFER', self.Id, PropName, Set)

    def _GetBytesPerSecond(self):
        return int(self._Property('BYTESPERSECOND'))

    BytesPerSecond = property(_GetBytesPerSecond,
    doc="""Transfer speed in bytes per second.

    :type: int
    """)

    def _GetBytesTransferred(self):
        return long(self._Property('BYTESTRANSFERRED'))

    BytesTransferred = property(_GetBytesTransferred,
    doc="""Number of bytes transferred.

    :type: long
    """)

    def _GetFailureReason(self):
        return str(self._Property('FAILUREREASON'))

    FailureReason = property(_GetFailureReason,
    doc="""Transfer failure reason.

    :type: `enums`.fileTransferFailureReason*
    """)

    def _GetFileName(self):
        return os.path.basename(self.FilePath)

    FileName = property(_GetFileName,
    doc="""Name of the transferred file.

    :type: str
    """)

    def _GetFilePath(self):
        return unicode2path(self._Property('FILEPATH'))

    FilePath = property(_GetFilePath,
    doc="""Full path to the transferred file.

    :type: str
    """)

    def _GetFileSize(self):
        return long(self._Property('FILESIZE'))

    FileSize = property(_GetFileSize,
    doc="""Size of the transferred file in bytes.

    :type: long
    """)

    def _GetFinishDatetime(self):
        from datetime import datetime
        return datetime.fromtimestamp(self.FinishTime)

    FinishDatetime = property(_GetFinishDatetime,
    doc="""File transfer end date and time.

    :type: datetime.datetime
    """)

    def _GetFinishTime(self):
        return float(self._Property('FINISHTIME'))

    FinishTime = property(_GetFinishTime,
    doc="""File transfer end timestamp.

    :type: float
    """)

    def _GetId(self):
        return self._Handle

    Id = property(_GetId,
    doc="""Unique file transfer Id.

    :type: int
    """)

    def _GetPartnerDisplayName(self):
        return self._Property('PARTNER_DISPNAME')

    PartnerDisplayName = property(_GetPartnerDisplayName,
    doc="""File transfer partner DisplayName.

    :type: unicode
    """)

    def _GetPartnerHandle(self):
        return str(self._Property('PARTNER_HANDLE'))

    PartnerHandle = property(_GetPartnerHandle,
    doc="""File transfer partner Skypename.

    :type: str
    """)

    def _GetStartDatetime(self):
        from datetime import datetime
        return datetime.fromtimestamp(self.StartTime)

    StartDatetime = property(_GetStartDatetime,
    doc="""File transfer start date and time.

    :type: datetime.datetime
    """)

    def _GetStartTime(self):
        return float(self._Property('STARTTIME'))

    StartTime = property(_GetStartTime,
    doc="""File transfer start timestamp.

    :type: float
    """)

    def _GetStatus(self):
        return str(self._Property('STATUS'))

    Status = property(_GetStatus,
    doc="""File transfer status.

    :type: `enums`.fileTransferStatus*
    """)

    def _GetType(self):
        return str(self._Property('TYPE'))

    Type = property(_GetType,
    doc="""File transfer type.

    :type: `enums`.fileTransferType*
    """)


class FileTransferCollection(CachedCollection):
    _CachedType = FileTransfer
